"""
example AHB data for 11042 as of 2021-10-01
"""
from uuid import UUID

from maus.models.anwendungshandbuch import AhbLine, AhbMetaInformation, FlatAnwendungshandbuch
from maus.models.message_implementation_guide import SegmentGroupHierarchy
from maus.navigation import AhbLocation, AhbLocationLayer, _DifferentialAhbLineHierarchyChange

example_sgh_11042 = SegmentGroupHierarchy(
    segment_group=None,
    opening_segment="UNH",
    sub_hierarchy=[
        SegmentGroupHierarchy(segment_group="SG1", opening_segment="RFF", sub_hierarchy=[]),
        SegmentGroupHierarchy(
            segment_group="SG2",
            opening_segment="NAD",
            sub_hierarchy=[SegmentGroupHierarchy(segment_group="SG3", opening_segment="CTA", sub_hierarchy=[])],
        ),
        SegmentGroupHierarchy(
            segment_group="SG4",
            opening_segment="IDE",
            sub_hierarchy=[
                SegmentGroupHierarchy(segment_group="SG5", opening_segment="LOC", sub_hierarchy=None),
                SegmentGroupHierarchy(segment_group="SG6", opening_segment="RFF", sub_hierarchy=None),
                SegmentGroupHierarchy(
                    segment_group="SG8",
                    opening_segment="SEQ",
                    sub_hierarchy=[
                        SegmentGroupHierarchy(segment_group="SG9", opening_segment="QTY", sub_hierarchy=None),
                        SegmentGroupHierarchy(segment_group="SG10", opening_segment="CCI", sub_hierarchy=None),
                    ],
                ),
                SegmentGroupHierarchy(segment_group="SG12", opening_segment="NAD", sub_hierarchy=None),
            ],
        ),
    ],
)

lines_locations_changes_11042 = [
    (
        AhbLine(
            guid=UUID("9cb133cb-ccde-43b0-9189-0a22f72ab125"),
            segment_group_key=None,
            segment_code="UNH",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Nachrichten-Kopfsegment",
            index=1,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
            segment_code="UNH",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("515e52ea-1a2a-4af9-a8f7-5f6d9f3a723c"),
            segment_group_key=None,
            segment_code="UNH",
            data_element="0062",
            value_pool_entry=None,
            name="Nachrichten-Referenznummer",
            ahb_expression="X",
            section_name="Nachrichten-Kopfsegment",
            index=2,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
            segment_code="UNH",
            data_element_id="0062",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("769a5391-1d3c-43a4-8e0e-0477e851082b"),
            segment_group_key=None,
            segment_code="UNH",
            data_element="0065",
            value_pool_entry="UTILMD",
            name="Netzanschluss-Stammdaten",
            ahb_expression="X",
            section_name="Nachrichten-Kopfsegment",
            index=3,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
            segment_code="UNH",
            data_element_id="0065",
            qualifier="UTILMD",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ea9571d9-285b-4675-91b0-1faf731bf494"),
            segment_group_key=None,
            segment_code="UNH",
            data_element="0052",
            value_pool_entry="D",
            name="Entwurfs-Version",
            ahb_expression="X",
            section_name="Nachrichten-Kopfsegment",
            index=4,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
            segment_code="UNH",
            data_element_id="0052",
            qualifier="D",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e4e38d7e-cd04-4c76-ac42-818726c66027"),
            segment_group_key=None,
            segment_code="UNH",
            data_element="0054",
            value_pool_entry="11A",
            name="Ausgabe 2011 - A",
            ahb_expression="X",
            section_name="Nachrichten-Kopfsegment",
            index=5,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
            segment_code="UNH",
            data_element_id="0054",
            qualifier="11A",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6e72ab5b-ef2e-4076-b933-4b0b8e567a76"),
            segment_group_key=None,
            segment_code="UNH",
            data_element="0051",
            value_pool_entry="UN",
            name="UN/CEFACT",
            ahb_expression="X",
            section_name="Nachrichten-Kopfsegment",
            index=6,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
            segment_code="UNH",
            data_element_id="0051",
            qualifier="UN",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("876f827f-e2e6-4b9d-b90a-504eff5ffb2e"),
            segment_group_key=None,
            segment_code="UNH",
            data_element="0057",
            value_pool_entry="5.2d",
            name="Versionsnummer derzugrundeliegendenBDEW-Nachrichtenbeschreibung",
            ahb_expression="X",
            section_name="Nachrichten-Kopfsegment",
            index=7,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
            segment_code="UNH",
            data_element_id="0057",
            qualifier="5.2d",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("aaa4708e-a494-488a-b386-c76bd4df49d2"),
            segment_group_key=None,
            segment_code="BGM",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Beginn der Nachricht",
            index=9,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
            segment_code="BGM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bd65ea9c-3ee9-4739-a57f-0c28ca94c26c"),
            segment_group_key=None,
            segment_code="BGM",
            data_element="1001",
            value_pool_entry="E01",
            name="Anmeldungen",
            ahb_expression="X",
            section_name="Beginn der Nachricht",
            index=10,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
            segment_code="BGM",
            data_element_id="1001",
            qualifier="E01",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f0cefec3-9bbe-4d42-8399-17db51ecd66e"),
            segment_group_key=None,
            segment_code="BGM",
            data_element="1004",
            value_pool_entry=None,
            name="Dokumentennummer",
            ahb_expression="X",
            section_name="Beginn der Nachricht",
            index=11,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
            segment_code="BGM",
            data_element_id="1004",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b77fb707-f380-4e39-b451-c6d5988db753"),
            segment_group_key=None,
            segment_code="DTM",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Nachrichtendatum",
            index=13,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
            segment_code="DTM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("873bc47b-8448-40b7-b400-2b93885ee9ac"),
            segment_group_key=None,
            segment_code="DTM",
            data_element="2005",
            value_pool_entry="137",
            name="Dokumenten-/Nachrichtendatum/-zeit",
            ahb_expression="X",
            section_name="Nachrichtendatum",
            index=14,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
            segment_code="DTM",
            data_element_id="2005",
            qualifier="137",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("14e8c009-63fb-4ae3-bae0-827268745cdc"),
            segment_group_key=None,
            segment_code="DTM",
            data_element="2380",
            value_pool_entry=None,
            name="Datum oder Uhrzeit oderZeitspanne, Wert",
            ahb_expression="X [931] [494]",
            section_name="Nachrichtendatum",
            index=15,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
            segment_code="DTM",
            data_element_id="2380",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("68b2eef5-226b-4770-b3da-c9feaa7c7e8f"),
            segment_group_key=None,
            segment_code="DTM",
            data_element="2379",
            value_pool_entry="303",
            name="CCYYMMDDHHMMZZZ",
            ahb_expression="X",
            section_name="Nachrichtendatum",
            index=16,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
            segment_code="DTM",
            data_element_id="2379",
            qualifier="303",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e3b7ff41-d82c-4423-ba04-d5e664895cac"),
            segment_group_key="SG2",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="MP-ID Absender",
            index=18,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("42548d91-b936-4646-a1c7-071d75ce72d4"),
            segment_group_key="SG2",
            segment_code="NAD",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="MP-ID Absender",
            index=19,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("387f6df2-45e8-48c7-9642-941cc5df65a5"),
            segment_group_key="SG2",
            segment_code="NAD",
            data_element="3035",
            value_pool_entry="MS",
            name="Dokumenten-/Nachrichtenausstellerbzw. -absender",
            ahb_expression="X",
            section_name="MP-ID Absender",
            index=20,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            ],
            segment_code="NAD",
            data_element_id="3035",
            qualifier="MS",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b910f3eb-fc13-4665-a62c-1dcb36f7775b"),
            segment_group_key="SG2",
            segment_code="NAD",
            data_element="3039",
            value_pool_entry=None,
            name="MP-ID",
            ahb_expression="X",
            section_name="MP-ID Absender",
            index=21,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            ],
            segment_code="NAD",
            data_element_id="3039",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9f34ddf5-ff05-41bc-87cf-4ce6a9b93334"),
            segment_group_key="SG2",
            segment_code="NAD",
            data_element="3055",
            value_pool_entry="9",
            name="GS1",
            ahb_expression="X",
            section_name="MP-ID Absender",
            index=22,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            ],
            segment_code="NAD",
            data_element_id="3055",
            qualifier="9",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("72223c29-19f7-475b-b31f-cee85f438d5d"),
            segment_group_key="SG2",
            segment_code="NAD",
            data_element="3055",
            value_pool_entry="293",
            name="DE, BDEW (Bundesverband der Energie- und Wasserwirtschaft e.V.)",
            ahb_expression="X",
            section_name="MP-ID Absender",
            index=23,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            ],
            segment_code="NAD",
            data_element_id="3055",
            qualifier="293",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1b48f3d5-84e0-46a6-9823-6731fbe2bd2b"),
            segment_group_key="SG2",
            segment_code="NAD",
            data_element="3055",
            value_pool_entry="332",
            name="DE, DVGW Service & Consult GmbH",
            ahb_expression="X",
            section_name="MP-ID Absender",
            index=24,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            ],
            segment_code="NAD",
            data_element_id="3055",
            qualifier="332",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1beb6a69-aa6e-4b45-a22b-432aaec7ef9e"),
            segment_group_key="SG3",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Kann",
            section_name="Ansprechpartner",
            index=26,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
                AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
            ],
            segment_code="CTA",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("9df97d33-ec9e-4959-8dd1-843b6d3152a1"),
            segment_group_key="SG3",
            segment_code="CTA",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Ansprechpartner",
            index=27,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
                AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
            ],
            segment_code="CTA",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("70455cbc-c050-4a2d-a98c-f54ee82771a2"),
            segment_group_key="SG3",
            segment_code="CTA",
            data_element="3139",
            value_pool_entry="IC",
            name="Informationskontakt",
            ahb_expression="X",
            section_name="Ansprechpartner",
            index=28,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
                AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
            ],
            segment_code="CTA",
            data_element_id="3139",
            qualifier="IC",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("00068e8c-fad8-404d-854f-a636e91a8514"),
            segment_group_key="SG3",
            segment_code="CTA",
            data_element="3412",
            value_pool_entry=None,
            name="Name vom Ansprechpartner",
            ahb_expression="X",
            section_name="Ansprechpartner",
            index=29,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
                AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
            ],
            segment_code="CTA",
            data_element_id="3412",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("765020b7-6a1c-4499-a66b-ffbf53981d0a"),
            segment_group_key="SG3",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Kommunikationsverbindung",
            index=31,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
                AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
            ],
            segment_code="COM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a5b189a0-7fbb-4230-a11a-ef1587c93850"),
            segment_group_key="SG3",
            segment_code="COM",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Kommunikationsverbindung",
            index=32,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
                AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
            ],
            segment_code="COM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e383aba5-1c1f-484c-953e-62344087a48f"),
            segment_group_key="SG3",
            segment_code="COM",
            data_element="3148",
            value_pool_entry=None,
            name="Nummer / Adresse",
            ahb_expression="X",
            section_name="Kommunikationsverbindung",
            index=33,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
                AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
            ],
            segment_code="COM",
            data_element_id="3148",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a1105e52-0e80-4d2f-8985-705196a1c024"),
            segment_group_key="SG3",
            segment_code="COM",
            data_element="3155",
            value_pool_entry="EM",
            name="Elektronische Post",
            ahb_expression="X [1P0..1]",
            section_name="Kommunikationsverbindung",
            index=34,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
                AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
            ],
            segment_code="COM",
            data_element_id="3155",
            qualifier="EM",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("267b239a-72fc-4510-b437-89966cfeacc4"),
            segment_group_key="SG3",
            segment_code="COM",
            data_element="3155",
            value_pool_entry="FX",
            name="Telefax",
            ahb_expression="X [1P0..1]",
            section_name="Kommunikationsverbindung",
            index=35,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
                AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
            ],
            segment_code="COM",
            data_element_id="3155",
            qualifier="FX",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2b8b2b61-5947-40cc-a6bb-86c622b95de4"),
            segment_group_key="SG3",
            segment_code="COM",
            data_element="3155",
            value_pool_entry="TE",
            name="Telefon",
            ahb_expression="X [1P0..1]",
            section_name="Kommunikationsverbindung",
            index=36,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
                AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
            ],
            segment_code="COM",
            data_element_id="3155",
            qualifier="TE",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("fe08b6f3-4502-4356-9bf0-04ed75e54c33"),
            segment_group_key="SG3",
            segment_code="COM",
            data_element="3155",
            value_pool_entry="AJ",
            name="weiteres Telefon",
            ahb_expression="X [1P0..1]",
            section_name="Kommunikationsverbindung",
            index=37,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
                AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
            ],
            segment_code="COM",
            data_element_id="3155",
            qualifier="AJ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5a08b58d-49c9-49fb-be1d-04f32a12e8cc"),
            segment_group_key="SG3",
            segment_code="COM",
            data_element="3155",
            value_pool_entry="AL",
            name="Handy",
            ahb_expression="X [1P0..1]",
            section_name="Kommunikationsverbindung",
            index=38,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
                AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
            ],
            segment_code="COM",
            data_element_id="3155",
            qualifier="AL",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("424e6e72-e43e-4832-ba8e-4730411b1a9e"),
            segment_group_key="SG2",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="MP-ID Empfänger",
            index=40,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MR"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("f5cb275c-af43-4c19-a8f0-9dab18fe6b74"),
            segment_group_key="SG2",
            segment_code="NAD",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="MP-ID Empfänger",
            index=41,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MR"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a2215938-8251-47fc-bda0-9a7f8b075348"),
            segment_group_key="SG2",
            segment_code="NAD",
            data_element="3035",
            value_pool_entry="MR",
            name="Nachrichtenempfänger",
            ahb_expression="X",
            section_name="MP-ID Empfänger",
            index=42,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MR"),
            ],
            segment_code="NAD",
            data_element_id="3035",
            qualifier="MR",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bbb2997a-bc8c-4a45-8139-fc4a7f222332"),
            segment_group_key="SG2",
            segment_code="NAD",
            data_element="3039",
            value_pool_entry=None,
            name="MP-ID",
            ahb_expression="X",
            section_name="MP-ID Empfänger",
            index=43,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MR"),
            ],
            segment_code="NAD",
            data_element_id="3039",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("44f2f1dd-d7a1-426b-822b-e32e409521d1"),
            segment_group_key="SG2",
            segment_code="NAD",
            data_element="3055",
            value_pool_entry="9",
            name="GS1",
            ahb_expression="X",
            section_name="MP-ID Empfänger",
            index=44,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MR"),
            ],
            segment_code="NAD",
            data_element_id="3055",
            qualifier="9",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("087160d1-cfd8-420c-9b9e-07269e9b6f1d"),
            segment_group_key="SG2",
            segment_code="NAD",
            data_element="3055",
            value_pool_entry="293",
            name="DE, BDEW (Bundesverband der Energie- und Wasserwirtschaft e.V.)",
            ahb_expression="X",
            section_name="MP-ID Empfänger",
            index=45,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MR"),
            ],
            segment_code="NAD",
            data_element_id="3055",
            qualifier="293",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("04cdfbae-db97-4dd3-a710-84cc326f2093"),
            segment_group_key="SG2",
            segment_code="NAD",
            data_element="3055",
            value_pool_entry="332",
            name="DE, DVGW Service & Consult GmbH",
            ahb_expression="X",
            section_name="MP-ID Empfänger",
            index=46,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MR"),
            ],
            segment_code="NAD",
            data_element_id="3055",
            qualifier="332",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5f60bb98-9c2a-4430-b6c0-1bc211788c00"),
            segment_group_key="SG4",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Vorgang",
            index=48,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="IDE",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("8e869ad5-6e82-4c05-a2d3-66a961b00ca6"),
            segment_group_key="SG4",
            segment_code="IDE",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Vorgang",
            index=49,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="IDE",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("73e8e68b-916f-43b6-8143-7e493eb86e0e"),
            segment_group_key="SG4",
            segment_code="IDE",
            data_element="7495",
            value_pool_entry="24",
            name="Transaktion",
            ahb_expression="X",
            section_name="Vorgang",
            index=50,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="IDE",
            data_element_id="7495",
            qualifier="24",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a04684ad-4203-408e-8000-7e3bfbc8456a"),
            segment_group_key="SG4",
            segment_code="IDE",
            data_element="7402",
            value_pool_entry=None,
            name="Vorgangsnummer",
            ahb_expression="X",
            section_name="Vorgang",
            index=51,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="IDE",
            data_element_id="7402",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("fc98bd9f-daf4-4c2d-b288-a5f31861d7b1"),
            segment_group_key="SG4",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Datum zum geplanten Leistungsbeginn",
            index=53,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="DTM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f0065249-c2fa-4e76-a618-7a44f366492c"),
            segment_group_key="SG4",
            segment_code="DTM",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Datum zum geplanten Leistungsbeginn",
            index=54,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="DTM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("30f91daf-13a9-47a9-bb83-9db8b0cc497b"),
            segment_group_key="SG4",
            segment_code="DTM",
            data_element="2005",
            value_pool_entry="76",
            name="Lieferdatum/-zeit, geplant",
            ahb_expression="X",
            section_name="Datum zum geplanten Leistungsbeginn",
            index=55,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="DTM",
            data_element_id="2005",
            qualifier="76",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c3a3da96-1a14-44b1-975d-9157c7db46ed"),
            segment_group_key="SG4",
            segment_code="DTM",
            data_element="2380",
            value_pool_entry=None,
            name="Datum oder Uhrzeit oderZeitspanne, Wert",
            ahb_expression="X [UB3]",
            section_name="Datum zum geplanten Leistungsbeginn",
            index=56,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="DTM",
            data_element_id="2380",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0c200005-1f1c-455f-bfcd-ad1d2125ac05"),
            segment_group_key="SG4",
            segment_code="DTM",
            data_element="2379",
            value_pool_entry="303",
            name="CCYYMMDDHHMMZZZ",
            ahb_expression="X",
            section_name="Datum zum geplanten Leistungsbeginn",
            index=57,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="DTM",
            data_element_id="2379",
            qualifier="303",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("159c7c41-4dd4-4299-a0a6-189517509450"),
            segment_group_key="SG4",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Transaktionsgrund",
            index=59,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("dd18dc04-747a-4367-9467-85779b4fd750"),
            segment_group_key="SG4",
            segment_code="STS",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss [61]",
            section_name="Transaktionsgrund",
            index=60,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7a1e10c3-8cd4-49b6-86e1-57848ff325ea"),
            segment_group_key="SG4",
            segment_code="STS",
            data_element="9015",
            value_pool_entry="7",
            name="Transaktionsgrund",
            ahb_expression="X",
            section_name="Transaktionsgrund",
            index=61,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id="9015",
            qualifier="7",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ac66691b-7e1c-4fe8-b766-ac30af1b39b9"),
            segment_group_key="SG4",
            segment_code="STS",
            data_element="9013",
            value_pool_entry="E01",
            name="Ein-/Auszug (Umzug)",
            ahb_expression="X",
            section_name="Transaktionsgrund",
            index=62,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id="9013",
            qualifier="E01",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4b5ebeac-994a-4388-b4be-a12964357d98"),
            segment_group_key="SG4",
            segment_code="STS",
            data_element="9013",
            value_pool_entry="E02",
            name="Einzug/Neuanlage",
            ahb_expression="X",
            section_name="Transaktionsgrund",
            index=63,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id="9013",
            qualifier="E02",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("693de653-0655-4a38-9969-f4330a6438e0"),
            segment_group_key="SG4",
            segment_code="STS",
            data_element="9013",
            value_pool_entry="E03",
            name="Wechsel",
            ahb_expression="X",
            section_name="Transaktionsgrund",
            index=64,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id="9013",
            qualifier="E03",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("de0c148f-b7a2-452f-8522-0010bb6c331f"),
            segment_group_key="SG4",
            segment_code="STS",
            data_element="9013",
            value_pool_entry="ZJ4",
            name="Übernahme aufgrund nicht erfolgtem iMS- Einbau",
            ahb_expression="X",
            section_name="Transaktionsgrund",
            index=65,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id="9013",
            qualifier="ZJ4",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("596116d2-a456-48b1-8d91-15defd3f329e"),
            segment_group_key="SG4",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Status der Antwort",
            index=67,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("578fb495-9b90-4c23-9752-83c04b74a821"),
            segment_group_key="SG4",
            segment_code="STS",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Status der Antwort",
            index=68,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("71a491e4-c5d7-4c5c-9021-c1d8e18e62b9"),
            segment_group_key="SG4",
            segment_code="STS",
            data_element="9015",
            value_pool_entry="E01",
            name="Status der Antwort",
            ahb_expression=None,
            section_name="Status der Antwort",
            index=69,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id="9015",
            qualifier="E01",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c36d8d67-d09d-4117-9eee-41a937b809a1"),
            segment_group_key="SG4",
            segment_code="STS",
            data_element="9013",
            value_pool_entry=None,
            name="Code des Prüfschritts",
            ahb_expression=None,
            section_name="Status der Antwort",
            index=70,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id="9013",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c5ea102a-f61c-477d-a3a6-d94fa5f2d388"),
            segment_group_key="SG4",
            segment_code="STS",
            data_element="1131",
            value_pool_entry="G_0053",
            name="Codeliste Gas Nr. G_0053",
            ahb_expression=None,
            section_name="Status der Antwort",
            index=71,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id="1131",
            qualifier="G_0053",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2868ea28-fdf3-49bb-8c62-952f61632bee"),
            segment_group_key="SG4",
            segment_code="STS",
            data_element="1131",
            value_pool_entry="G_0054",
            name="Codeliste Gas Nr. G_0054",
            ahb_expression=None,
            section_name="Status der Antwort",
            index=72,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id="1131",
            qualifier="G_0054",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("52aaed89-feb2-4649-9632-a3b645e6fcbf"),
            segment_group_key="SG4",
            segment_code="STS",
            data_element="1131",
            value_pool_entry="Codeliste Strom Nr. S_0055",
            name="S_0055",
            ahb_expression=None,
            section_name="Status der Antwort",
            index=73,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id="1131",
            qualifier="Codeliste Strom Nr. S_0055",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d67c5593-f757-4a64-ae11-3f99ee6a8b55"),
            segment_group_key="SG4",
            segment_code="STS",
            data_element="1131",
            value_pool_entry="Codeliste Strom Nr. S_0056",
            name="S_0056",
            ahb_expression=None,
            section_name="Status der Antwort",
            index=74,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="STS",
            data_element_id="1131",
            qualifier="Codeliste Strom Nr. S_0056",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c74ac745-fe0f-4c2f-9820-5bc6fb347808"),
            segment_group_key="SG4",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Beauftragung oder Beendigung liegt vor",
            index=76,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="AGR",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ab8c3c63-7cf1-4d29-9fe3-b541ad047b8b"),
            segment_group_key="SG4",
            segment_code="AGR",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Kann",
            section_name="Beauftragung oder Beendigung liegt vor",
            index=77,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="AGR",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("64515215-8d58-42a6-9f1a-a7aaa5f7645d"),
            segment_group_key="SG4",
            segment_code="AGR",
            data_element="7431",
            value_pool_entry="9",
            name="Vertrag",
            ahb_expression="X",
            section_name="Beauftragung oder Beendigung liegt vor",
            index=78,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="AGR",
            data_element_id="7431",
            qualifier="9",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3c69a397-8f1a-47ea-9817-f7120058e98f"),
            segment_group_key="SG4",
            segment_code="AGR",
            data_element="7433",
            value_pool_entry="Z04",
            name="Vertrag zwischen AN und MSB",
            ahb_expression="X [1P0..1]",
            section_name="Beauftragung oder Beendigung liegt vor",
            index=79,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="AGR",
            data_element_id="7433",
            qualifier="Z04",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3c83c95c-8c1c-4bdf-8168-2e2573479ab5"),
            segment_group_key="SG4",
            segment_code="AGR",
            data_element="7433",
            value_pool_entry="Z06",
            name="Vertragsbeendigung liegt vor",
            ahb_expression="X [1P0..1]",
            section_name="Beauftragung oder Beendigung liegt vor",
            index=80,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            ],
            segment_code="AGR",
            data_element_id="7433",
            qualifier="Z06",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c1f5b55b-b1d9-41c8-ab83-b5fa44e4eee2"),
            segment_group_key="SG5",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss ([77] u[78]) u (([61]u [584]) o[583])",
            section_name="Meldepunkt",
            index=82,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG5", opening_segment_code="LOC", opening_qualifier="172"),
            ],
            segment_code="LOC",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("c226852b-f723-49f0-83ba-d4f4a78274ef"),
            segment_group_key="SG5",
            segment_code="LOC",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Meldepunkt",
            index=83,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG5", opening_segment_code="LOC", opening_qualifier="172"),
            ],
            segment_code="LOC",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8b953e7f-d0d4-4a25-a10c-27a8ce7ae462"),
            segment_group_key="SG5",
            segment_code="LOC",
            data_element="3227",
            value_pool_entry="172",
            name="Meldepunkt",
            ahb_expression="X",
            section_name="Meldepunkt",
            index=84,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG5", opening_segment_code="LOC", opening_qualifier="172"),
            ],
            segment_code="LOC",
            data_element_id="3227",
            qualifier="172",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3aa7a329-f0b0-4479-80b9-5b5a02da0c9b"),
            segment_group_key="SG5",
            segment_code="LOC",
            data_element="3225",
            value_pool_entry=None,
            name="Identifikator",
            ahb_expression="X [953]",
            section_name="Meldepunkt",
            index=85,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG5", opening_segment_code="LOC", opening_qualifier="172"),
            ],
            segment_code="LOC",
            data_element_id="3225",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5d5d5cfc-5558-468a-8177-772bc33e40c4"),
            segment_group_key="SG6",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Prüfidentifikator",
            index=88,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z13"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("ddbbf0ff-3d81-491c-96dc-743f4d51dead"),
            segment_group_key="SG6",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Prüfidentifikator",
            index=89,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z13"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b60dac1f-6f6f-4794-b1d5-849573dc9fea"),
            segment_group_key="SG6",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z13",
            name="Prüfidentifikator",
            ahb_expression="X",
            section_name="Prüfidentifikator",
            index=90,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z13"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z13",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a15d8867-431f-46a6-bd20-1c06a0f67459"),
            segment_group_key="SG6",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry="11042",
            name="WiM / Anmeldung MSB",
            ahb_expression="X",
            section_name="Prüfidentifikator",
            index=91,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z13"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier="11042",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("abef9170-0bc4-4119-b2c1-4edd4f1a22e8"),
            segment_group_key="SG6",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry="11043",
            name="WiM / Bestätigung Anmeldung MSB",
            ahb_expression=None,
            section_name="Prüfidentifikator",
            index=92,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z13"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier="11043",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3aef88cb-a664-48ae-8b85-a4ea6ddc248d"),
            segment_group_key="SG6",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry="11044",
            name="WiM / Ablehnung Anmeldung MSB",
            ahb_expression=None,
            section_name="Prüfidentifikator",
            index=93,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z13"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier="11044",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f7c26003-13da-4bbf-a774-91deaa03f976"),
            segment_group_key="SG6",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz Vorgangsnummer (aus Anfragenachricht)",
            index=95,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="TN"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("8cfd0831-c530-4bc7-b7c3-b2a17a674841"),
            segment_group_key="SG6",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz Vorgangsnummer (aus Anfragenachricht)",
            index=96,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="TN"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6bc294f7-b10a-42d5-bffc-88135eb5efbf"),
            segment_group_key="SG6",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="TN",
            name="Transaktions-Referenznummer",
            ahb_expression=None,
            section_name="Referenz Vorgangsnummer (aus Anfragenachricht)",
            index=97,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="TN"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="TN",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d6f4256e-4ff5-4370-8084-5b912e0b45fd"),
            segment_group_key="SG6",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="Vorgangsnummer",
            ahb_expression=None,
            section_name="Referenz Vorgangsnummer (aus Anfragenachricht)",
            index=98,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="TN"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("769fece8-14df-4656-93ca-f5b9c833b428"),
            segment_group_key="SG6",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation für Termine der Marktlokation",
            index=100,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("d2f3090f-77a6-4f73-a57c-08a92ef98d92"),
            segment_group_key="SG6",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation für Termine der Marktlokation",
            index=101,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("63375810-1db6-482f-88ba-34b736ac9fdb"),
            segment_group_key="SG6",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z18",
            name="Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation für Termine der Marktlokation",
            index=102,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("eae34e55-d12b-4020-bbd9-b1a08f52df78"),
            segment_group_key="SG6",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID der Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation für Termine der Marktlokation",
            index=103,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e926691d-ce16-431a-8c88-83b78672277b"),
            segment_group_key="SG6",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Geplante Turnusablesung des MSB (Strom)",
            index=105,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("18cf539d-8178-425c-acf2-ff5419a8f5a8"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Geplante Turnusablesung des MSB (Strom)",
            index=106,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a92196d6-e947-403a-ae32-3a31a982c17e"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2005",
            value_pool_entry="752",
            name="Geplante Turnusablesung",
            ahb_expression=None,
            section_name="Geplante Turnusablesung des MSB (Strom)",
            index=108,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id="2005",
            qualifier="752",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6ff6a5e3-2815-4cef-822c-33cb1f375ea4"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2380",
            value_pool_entry=None,
            name="Datum oder Uhrzeit oderZeitspanne, Wert",
            ahb_expression=None,
            section_name="Geplante Turnusablesung des MSB (Strom)",
            index=109,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id="2380",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6c3c7e72-a396-4223-8d97-af33ed68de2f"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2379",
            value_pool_entry="104",
            name="MMWW-MMWW",
            ahb_expression=None,
            section_name="Geplante Turnusablesung des MSB (Strom)",
            index=110,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id="2379",
            qualifier="104",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2283024f-6bf6-4f57-8417-2fe90571451d"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2379",
            value_pool_entry="106",
            name="MMDD",
            ahb_expression=None,
            section_name="Geplante Turnusablesung des MSB (Strom)",
            index=111,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id="2379",
            qualifier="106",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("30fd3f52-482f-41dd-8225-12fd1dd6028c"),
            segment_group_key="SG6",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Turnusableseintervall des MSB (Strom)",
            index=113,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b3ec46d4-b4a6-4dab-9c72-7ae6e6eb09dd"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Turnusableseintervall des MSB (Strom)",
            index=114,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2799fa2a-73c9-43b6-8677-cedd87b45a83"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2005",
            value_pool_entry="672",
            name="zugewiesene Periode",
            ahb_expression=None,
            section_name="Turnusableseintervall des MSB (Strom)",
            index=115,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id="2005",
            qualifier="672",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6638f635-c38a-466d-a54e-d594085d41e4"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2380",
            value_pool_entry=None,
            name="Datum oder Uhrzeit oderZeitspanne, Wert",
            ahb_expression=None,
            section_name="Turnusableseintervall des MSB (Strom)",
            index=116,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id="2380",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e56a3cd5-1963-4ab2-973f-fda4ee2094aa"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2379",
            value_pool_entry="802",
            name="Monat",
            ahb_expression=None,
            section_name="Turnusableseintervall des MSB (Strom)",
            index=117,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id="2379",
            qualifier="802",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("03eeab6e-4136-408d-be5b-5567c3d65823"),
            segment_group_key="SG6",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Abrechnungsintervall des LF",
            index=119,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("75b9529e-5bea-4a30-92dd-ad59bc116837"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Abrechnungsintervall des LF",
            index=120,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d1538df9-50c0-4ed3-aed6-5976c4d2b22c"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2005",
            value_pool_entry="Z20",
            name="Abrechnungsintervall desLF",
            ahb_expression=None,
            section_name="Abrechnungsintervall des LF",
            index=121,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id="2005",
            qualifier="Z20",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6674a99c-36cc-4b47-8cc6-478e5503e4e7"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2380",
            value_pool_entry=None,
            name="Datum oder Uhrzeit oderZeitspanne, Wert",
            ahb_expression=None,
            section_name="Abrechnungsintervall des LF",
            index=122,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id="2380",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("73f81051-6ba4-4875-8924-5004ee7bdd5e"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2379",
            value_pool_entry="802",
            name="Monat",
            ahb_expression=None,
            section_name="Abrechnungsintervall des LF",
            index=123,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
            ],
            segment_code="DTM",
            data_element_id="2379",
            qualifier="802",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d498e94c-b05b-4e94-bc4c-681c65c50f32"),
            segment_group_key="SG6",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Messlokation für Termine der Messlokation",
            index=125,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("7ec6fe59-4757-4e4f-a85c-9daa5155b2c3"),
            segment_group_key="SG6",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Messlokation für Termine der Messlokation",
            index=126,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c04d0419-dc37-47db-9c4d-09627ccec456"),
            segment_group_key="SG6",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z19",
            name="Messlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Messlokation für Termine der Messlokation",
            index=127,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z19",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("575a80a0-e5b0-4219-a3e9-03ae3aa33fd7"),
            segment_group_key="SG6",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID der Messlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Messlokation für Termine der Messlokation",
            index=128,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("61a4bcef-e6cd-4b86-b1f9-a56421ee4c9e"),
            segment_group_key="SG6",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Geplante Turnusablesung des NB (Gas)",
            index=130,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="DTM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8b20e64b-36c1-4ba4-8678-c298e0a4d108"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Geplante Turnusablesung des NB (Gas)",
            index=131,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="DTM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e0f1acd7-c6a4-4518-901e-e822c84f078d"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2005",
            value_pool_entry="752",
            name="Geplante Turnusablesung",
            ahb_expression=None,
            section_name="Geplante Turnusablesung des NB (Gas)",
            index=132,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="DTM",
            data_element_id="2005",
            qualifier="752",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("57a8c02b-f7bd-4c73-9285-c6bfc8a9ea26"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2380",
            value_pool_entry=None,
            name="Datum oder Uhrzeit oderZeitspanne, Wert",
            ahb_expression=None,
            section_name="Geplante Turnusablesung des NB (Gas)",
            index=133,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="DTM",
            data_element_id="2380",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b1814f43-b4d8-4508-ad62-834ae69d4da6"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2379",
            value_pool_entry="104",
            name="MMWW-MMWW",
            ahb_expression=None,
            section_name="Geplante Turnusablesung des NB (Gas)",
            index=134,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="DTM",
            data_element_id="2379",
            qualifier="104",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("11081411-b2f5-44f3-955d-f20aa4d40d41"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2379",
            value_pool_entry="106",
            name="MMDD",
            ahb_expression=None,
            section_name="Geplante Turnusablesung des NB (Gas)",
            index=135,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="DTM",
            data_element_id="2379",
            qualifier="106",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c5661a0c-addf-4985-964f-e2722b322482"),
            segment_group_key="SG6",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Turnusableseintervall des NB (Gas)",
            index=137,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="DTM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("41dbfb40-4fef-4d4c-ac68-71248789db93"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Turnusableseintervall des NB (Gas)",
            index=138,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="DTM",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("28f8fa11-b654-414b-961e-7e74bbbdc526"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2005",
            value_pool_entry="672",
            name="zugewiesene Periode",
            ahb_expression=None,
            section_name="Turnusableseintervall des NB (Gas)",
            index=139,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="DTM",
            data_element_id="2005",
            qualifier="672",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e316f9f0-1d60-4708-bfe2-8bd2298ef054"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2380",
            value_pool_entry=None,
            name="Datum oder Uhrzeit oderZeitspanne, Wert",
            ahb_expression=None,
            section_name="Turnusableseintervall des NB (Gas)",
            index=140,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="DTM",
            data_element_id="2380",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8bb1e13a-1898-42ab-b0b1-0ef3678cdd6c"),
            segment_group_key="SG6",
            segment_code="DTM",
            data_element="2379",
            value_pool_entry="802",
            name="Monat",
            ahb_expression=None,
            section_name="Turnusableseintervall des NB (Gas)",
            index=141,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
            ],
            segment_code="DTM",
            data_element_id="2379",
            qualifier="802",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("570549d3-9f2d-4fbc-b0eb-a2c097ca0bd4"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Daten der Marktlokation",
            index=143,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("acf72543-9fb4-447c-91c4-bbd74684cc87"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Daten der Marktlokation",
            index=144,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2be55951-649f-4491-8afd-6fd79b4fcb22"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z01",
            name="Daten der Marktlokation",
            ahb_expression=None,
            section_name="Daten der Marktlokation",
            index=145,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z01",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f56eadcf-8ae6-4646-931b-b104c5162874"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=147,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1dcfb3d4-65c3-446b-9ba3-7f8627ce19c4"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=148,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c7ca939a-e10b-4253-ac30-0307a38e3850"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z18",
            name="Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=149,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("aa1de3e9-a6e3-4147-aabe-27f4ede62210"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID der Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=150,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ea79ed4b-182f-45bd-b4f9-b6eba5b962d9"),
            segment_group_key="SG9",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Arbeit / Leistung für tagesparameterabhängige Marktlokation",
            index=152,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="265"),
            ],
            segment_code="QTY",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("1de37d12-ef15-480a-8860-0da840b57998"),
            segment_group_key="SG9",
            segment_code="QTY",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Arbeit / Leistung für tagesparameterabhängige Marktlokation",
            index=153,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="265"),
            ],
            segment_code="QTY",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2807369a-f7d8-4e81-8640-e46e35b40651"),
            segment_group_key="SG9",
            segment_code="QTY",
            data_element="6063",
            value_pool_entry="265",
            name="Veranschlagte Jahresmenge Gesamt/ Jahresverbrauchsprogno se für spezifische Arbeit für tagesparameterabhängig e Marktlokation",
            ahb_expression=None,
            section_name="Arbeit / Leistung für tagesparameterabhängige Marktlokation",
            index=154,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="265"),
            ],
            segment_code="QTY",
            data_element_id="6063",
            qualifier="265",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("628824a1-8a47-40e9-9438-844cc5ffaf78"),
            segment_group_key="SG9",
            segment_code="QTY",
            data_element="6063",
            value_pool_entry="Z08",
            name='Angepasste elektrische Arbeit nach Anhang D der VDN-Richtlinie "Lastprofile für unterbrechbare Verbrauchseinrichtungen"',
            ahb_expression=None,
            section_name="Arbeit / Leistung für tagesparameterabhängige Marktlokation",
            index=155,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="265"),
            ],
            segment_code="QTY",
            data_element_id="6063",
            qualifier="Z08",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5f4ceb61-1857-4057-8ef5-b4eb69a29a45"),
            segment_group_key="SG9",
            segment_code="QTY",
            data_element="6060",
            value_pool_entry=None,
            name="Menge",
            ahb_expression=None,
            section_name="Arbeit / Leistung für tagesparameterabhängige Marktlokation",
            index=156,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="265"),
            ],
            segment_code="QTY",
            data_element_id="6060",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0cc8ae4d-aa07-4e57-a273-0509861174a0"),
            segment_group_key="SG9",
            segment_code="QTY",
            data_element="6411",
            value_pool_entry="Z16",
            name="kWh/K (Kilowatt- Stunde/Kelvin)",
            ahb_expression=None,
            section_name="Arbeit / Leistung für tagesparameterabhängige Marktlokation",
            index=157,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="265"),
            ],
            segment_code="QTY",
            data_element_id="6411",
            qualifier="Z16",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ff2f1555-f1e5-4c33-ab52-f54df8bb4e45"),
            segment_group_key="SG9",
            segment_code="QTY",
            data_element="6411",
            value_pool_entry="KWH",
            name="Kilowattstunde",
            ahb_expression=None,
            section_name="Arbeit / Leistung für tagesparameterabhängige Marktlokation",
            index=158,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="265"),
            ],
            segment_code="QTY",
            data_element_id="6411",
            qualifier="KWH",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2ece77de-e6cd-4aea-bdfc-f9b2864aa289"),
            segment_group_key="SG9",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Veranschlagte Jahresmenge gesamt",
            index=160,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="31"),
            ],
            segment_code="QTY",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("ce40a200-6be4-4768-b77c-44a960e8d1ff"),
            segment_group_key="SG9",
            segment_code="QTY",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Veranschlagte Jahresmenge gesamt",
            index=161,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="31"),
            ],
            segment_code="QTY",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3e837cc8-e48b-4da7-b82e-910956d7c396"),
            segment_group_key="SG9",
            segment_code="QTY",
            data_element="6063",
            value_pool_entry="31",
            name="VeranschlagteJahresmenge Gesamt",
            ahb_expression=None,
            section_name="Veranschlagte Jahresmenge gesamt",
            index=162,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="31"),
            ],
            segment_code="QTY",
            data_element_id="6063",
            qualifier="31",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f40dbf11-61cd-4286-a96b-808eba0456fb"),
            segment_group_key="SG9",
            segment_code="QTY",
            data_element="6060",
            value_pool_entry=None,
            name="Mengenangabe",
            ahb_expression=None,
            section_name="Veranschlagte Jahresmenge gesamt",
            index=163,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="31"),
            ],
            segment_code="QTY",
            data_element_id="6060",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("eeccb280-2c3b-4aa0-88ee-dfd97030b6bd"),
            segment_group_key="SG9",
            segment_code="QTY",
            data_element="6411",
            value_pool_entry="KWH",
            name="Kilowattstunde",
            ahb_expression=None,
            section_name="Veranschlagte Jahresmenge gesamt",
            index=165,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="31"),
            ],
            segment_code="QTY",
            data_element_id="6411",
            qualifier="KWH",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e2ded2fe-6505-4664-bfab-e4e24f84613a"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Lieferrichtung",
            index=167,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z30"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("b1df7c0a-3288-4fb5-8a7b-7924435aa6f6"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Lieferrichtung",
            index=168,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z30"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e612a141-a821-42ed-8769-cc3e3dcaa785"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z30",
            name="Lieferrichtung",
            ahb_expression=None,
            section_name="Lieferrichtung",
            index=169,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z30"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z30",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("426d64e5-e293-47bc-bd17-bf62f85b62f5"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z06",
            name="Erzeugung",
            ahb_expression=None,
            section_name="Lieferrichtung",
            index=170,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z30"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z06",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("93408fea-0b35-4f9b-be96-3c1437a8c62a"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z07",
            name="Verbrauch",
            ahb_expression=None,
            section_name="Lieferrichtung",
            index=171,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z30"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z07",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3beabb56-3267-441a-a819-9892e084d116"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zugeordnete Marktpartner",
            index=173,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("10e8abdb-0577-43c2-be8d-25190f3315ac"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zugeordnete Marktpartner",
            index=174,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("242c6ebd-1953-4537-8ee0-049cdc881177"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZB3",
            name="ZugeordneterMarktpartner",
            ahb_expression=None,
            section_name="Zugeordnete Marktpartner",
            index=175,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZB3",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1336c7c2-ba3d-4897-b867-599a5e0e7055"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Lieferant",
            index=177,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5ddb6304-efd2-44f6-b0c0-e08a632d914e"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Lieferant",
            index=178,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5554ddc2-9c54-47e5-ba70-6ea8095ce7f4"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z89",
            name="LF",
            ahb_expression=None,
            section_name="Lieferant",
            index=179,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z89",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("19fbf97c-a004-4b71-a90c-b7296c951d17"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="1131",
            value_pool_entry=None,
            name="MP-ID",
            ahb_expression=None,
            section_name="Lieferant",
            index=180,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="1131",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1f2cd729-c67d-43fc-b670-4d515112f9d3"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Übertragungsnetzbetreiber",
            index=182,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ecc9bb2c-0def-43d0-9b8f-473f6836143e"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Übertragungsnetzbetreiber",
            index=183,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1a7538f7-38b5-40e6-843e-18ca676fada2"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z90",
            name="ÜNB",
            ahb_expression=None,
            section_name="Übertragungsnetzbetreiber",
            index=184,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z90",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("68450e9f-9502-4e91-8bf0-82689158c310"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="1131",
            value_pool_entry=None,
            name="MP-ID",
            ahb_expression=None,
            section_name="Übertragungsnetzbetreiber",
            index=185,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="1131",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("895f028e-bb17-4303-b19b-01ca6f513000"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Messstellenbetreiber",
            index=187,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4c6a7568-e2b9-42a0-a5e3-26b2cf9fa33b"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Messstellenbetreiber",
            index=188,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("681b4509-3efa-42ec-9bf7-be6b427869d5"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z91",
            name="MSB",
            ahb_expression=None,
            section_name="Messstellenbetreiber",
            index=189,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z91",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6c60ba82-9f58-4302-bfe3-eb8c3f0921a5"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="1131",
            value_pool_entry=None,
            name="MP-ID",
            ahb_expression=None,
            section_name="Messstellenbetreiber",
            index=190,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="1131",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b17c072b-42c5-4ac5-b1f2-61ae0f7369a7"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z19",
            name="Auf vertraglicher Grundlage gegenüber Anschlussnutzer / Anschlussnehmer",
            ahb_expression=None,
            section_name="Messstellenbetreiber",
            index=191,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z19",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e0059ae2-ce4c-40f0-af99-994e51be5e6f"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z20",
            name="In der Ausübung der Weiterverpflichtung durch den gMSB",
            ahb_expression=None,
            section_name="Messstellenbetreiber",
            index=192,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z20",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2387e17b-ff7c-4084-91ed-d2e057d9c27c"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Spannungsebene der Marktlokation",
            index=194,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("5d238395-eabd-4485-92d8-56f8f097c380"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Spannungsebene der Marktlokation",
            index=195,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("54985390-bf37-484b-81a9-78d1bc7f3e1c"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="E03",
            name="Spannungsebene derMarktlokation",
            ahb_expression=None,
            section_name="Spannungsebene der Marktlokation",
            index=196,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="E03",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5d271138-f31a-433c-a44c-bb0d249f1192"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Spannungsebene der Marktlokation",
            index=198,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7a11e8cd-9018-4d94-b5e7-a343284a7620"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Spannungsebene der Marktlokation",
            index=199,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("98bebbc4-271e-4995-b1cb-bae81140ccfa"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="E03",
            name="Höchstspannung",
            ahb_expression=None,
            section_name="Spannungsebene der Marktlokation",
            index=200,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="E03",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9202d1c9-4456-46de-ac2d-8f0fbfc90ee1"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="E04",
            name="Hochspannung",
            ahb_expression=None,
            section_name="Spannungsebene der Marktlokation",
            index=201,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="E04",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b3154ad9-2497-4867-bd5f-ebba9a087b37"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="E05",
            name="Mittelspannung",
            ahb_expression=None,
            section_name="Spannungsebene der Marktlokation",
            index=202,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="E05",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2ad47a04-7d8c-4fb8-bba3-479bc1a706e8"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="E06",
            name="Niederspannung",
            ahb_expression=None,
            section_name="Spannungsebene der Marktlokation",
            index=203,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="E06",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5147dae1-1bb5-43fd-a2b0-4cb804b4aa11"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Messtechnische Einordnung der Marktlokation",
            index=205,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z83"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("f44056be-43d6-4a89-bc3f-20d88d83cac0"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Messtechnische Einordnung der Marktlokation",
            index=206,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z83"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("48571d64-c89d-4fc5-8bb4-a4dc9467d294"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z83",
            name="MesstechnischeEinordnung derMarktlokation",
            ahb_expression=None,
            section_name="Messtechnische Einordnung der Marktlokation",
            index=207,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z83"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z83",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c1174c8f-8c62-4d4b-9969-314522ae28e1"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Messtechnische Einordnung der Marktlokation",
            index=209,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z83"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c099af44-92fb-4e7d-91be-e9d2152ef07e"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Messtechnische Einordnung der Marktlokation",
            index=210,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z83"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8a580c7a-0338-41bb-aa5a-e9a80ac4170a"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z52",
            name="iMS",
            ahb_expression=None,
            section_name="Messtechnische Einordnung der Marktlokation",
            index=211,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z83"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z52",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2a4352da-bd1a-4f14-b5a5-cb8001d3c965"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z53",
            name="kME / mME",
            ahb_expression=None,
            section_name="Messtechnische Einordnung der Marktlokation",
            index=212,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z83"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z53",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b9a08739-82bb-4edb-844e-ef8101cfa10f"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Aggregationsverantwortung in MaBiS",
            index=214,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="6"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("a92afea5-0fb4-4e4f-adeb-dad0ff7c74df"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Aggregationsverantwortung in MaBiS",
            index=215,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="6"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("16de1b31-90c9-4e6d-84b8-79cf62887a06"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="6",
            name="Verantwortlicher",
            ahb_expression=None,
            section_name="Aggregationsverantwortung in MaBiS",
            index=216,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="6"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="6",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("13a3f425-39d9-4f8b-818f-4b68612926a7"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZA8",
            name="NB",
            ahb_expression=None,
            section_name="Aggregationsverantwortung in MaBiS",
            index=217,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="6"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZA8",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("555896d3-7b4d-416c-8b8a-aa7a3dda414a"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZA9",
            name="ÜNB",
            ahb_expression=None,
            section_name="Aggregationsverantwortung in MaBiS",
            index=218,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="6"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZA9",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("53a85405-10a1-4ec7-90bf-3017e05fd314"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Prognosegrundlage der Marktlokation",
            index=220,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("2e51a38c-7db5-40d2-a0b2-beae58daaf6e"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Prognosegrundlage der Marktlokation",
            index=221,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ecbdb212-2923-4d87-85c0-a667ee7b62f7"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZC0",
            name="Prognose auf Basis von Werten",
            ahb_expression=None,
            section_name="Prognosegrundlage der Marktlokation",
            index=222,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZC0",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0c1fd4f0-d632-41e3-b84f-e33485c353ab"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZA6",
            name="Prognose auf Basis von Profilen",
            ahb_expression=None,
            section_name="Prognosegrundlage der Marktlokation",
            index=223,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZA6",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6e292fb7-ef2b-41ab-abe7-8936b07e6978"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Details der Prognosegrundlage",
            index=225,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("30218cf6-be70-4290-8787-4ab04a789f18"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Details der Prognosegrundlage",
            index=226,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9756c8a1-791a-4d38-9e78-69a6ca7d8d04"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="E02",
            name="SLP/SEP",
            ahb_expression=None,
            section_name="Details der Prognosegrundlage",
            index=228,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="E02",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6b3f14e7-a604-45d8-ba24-0644af4dfb81"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="E14",
            name="TLP/TEP",
            ahb_expression=None,
            section_name="Details der Prognosegrundlage",
            index=229,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="E14",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("00f6af6f-49f8-4ef7-8c6e-142065375946"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z36",
            name="TEP mit Referenzmessung",
            ahb_expression=None,
            section_name="Details der Prognosegrundlage",
            index=230,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z36",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0950fac8-a076-4d0a-8b01-ead19ad0ddab"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Art der erzeugenden Marktlokation",
            index=232,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("1b275cce-1e1a-4427-9dc1-2963e49f6e87"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Art der erzeugenden Marktlokation",
            index=233,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c8945272-a1b2-4015-bc08-c343247c635f"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z34",
            name="Art der erzeugendenMarktlokation",
            ahb_expression=None,
            section_name="Art der erzeugenden Marktlokation",
            index=234,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z34",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("28e8bb2a-856c-4776-84f2-ab0d0db3a97a"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Art der erzeugenden Marktlokation",
            index=236,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2b523651-c9fb-473f-a16e-adcd82c47bfd"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Art der erzeugenden Marktlokation",
            index=237,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b99187a6-e53a-412d-8508-63068cff1995"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z33",
            name="EEG-Marktlokation ohne DV-Pflicht",
            ahb_expression=None,
            section_name="Art der erzeugenden Marktlokation",
            index=238,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z33",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("98ce05da-51e3-4902-b7d8-88e9a9854e6a"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z34",
            name="KWKG-Marktlokation ohne DV-Pflicht",
            ahb_expression=None,
            section_name="Art der erzeugenden Marktlokation",
            index=239,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z34",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("666a53df-4a83-4172-9fdf-952d4dda6117"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z35",
            name="sonstige Marktlokation",
            ahb_expression=None,
            section_name="Art der erzeugenden Marktlokation",
            index=240,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z35",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9706df17-6a7e-4907-88eb-1f8a5d9ad883"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z37",
            name="EEG-Marktlokation mit DV-Pflicht",
            ahb_expression=None,
            section_name="Art der erzeugenden Marktlokation",
            index=241,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z37",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("26beb3f2-1166-413b-95ce-7b9ea007ecb7"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z46",
            name="KWKG-Marktlokation mit DV-Pflicht",
            ahb_expression=None,
            section_name="Art der erzeugenden Marktlokation",
            index=242,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z46",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0248128e-a256-4573-9508-026713a58726"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Versorgungsart der Marktlokation",
            index=244,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z36"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("41fe81e6-8003-422b-b3f1-b4bf124706e3"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Versorgungsart der Marktlokation",
            index=245,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z36"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("025846c2-1eb0-48b2-905d-757e4fd136fa"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z36",
            name="Versorgungsart derMarktlokation",
            ahb_expression=None,
            section_name="Versorgungsart der Marktlokation",
            index=246,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z36"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z36",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6a6f7af5-5672-4931-9b06-e06e5041ad77"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Erforderlicher Wert der Marktlokation",
            index=248,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("998621f0-8ad4-49c1-900d-d9de3e8f7568"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Erforderlicher Wert der Marktlokation",
            index=250,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1fe839f8-e20a-4e53-98b7-41b4b6107fa6"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z27",
            name="Erforderlicher Wert derMarktlokation",
            ahb_expression=None,
            section_name="Erforderlicher Wert der Marktlokation",
            index=251,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z27",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c3ad63af-0173-4ec4-a012-6b5009d5a602"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=253,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f2ce13f6-362f-4bba-8334-dc11cb7994fd"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=254,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d48bd687-5b4f-4da5-8c7a-f4e436c187fd"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z18",
            name="Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=255,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3bbdde8f-a027-4ea5-9851-cc162b30cca1"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID einer Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=256,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0d9d7c17-8d5a-4624-a0fa-3c0f495ccbe0"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Beschreibung erforderlicher Wert",
            index=258,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("d333e6fc-548d-43b0-afe0-1d6d2d4b074d"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Beschreibung erforderlicher Wert",
            index=259,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f32b44cb-ae9b-4443-a1a8-11dfd6c76aab"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z35",
            name="Beschreibungerforderlicher Wert",
            ahb_expression=None,
            section_name="Beschreibung erforderlicher Wert",
            index=260,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z35",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b74662d0-e36a-4eef-949c-82abf85f9b8a"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wert",
            index=262,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1577181b-0edc-4d13-8ee1-f2f1582bfc4b"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wert",
            index=263,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("616a1515-def3-48c5-819e-d79f69e98ab1"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC2",
            name="Wirkarbeit",
            ahb_expression=None,
            section_name="Wert",
            index=264,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC2",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("fdd77c74-c19c-4afb-8ed5-5f487f02cf10"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC3",
            name="Blindarbeit",
            ahb_expression=None,
            section_name="Wert",
            index=265,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC3",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5b7113b3-597c-4cfa-93bb-147ea736a5f1"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wertegranularität",
            index=267,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c3db7ec0-46f5-478d-92f7-377b9abb5552"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wertegranularität",
            index=268,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a05c6128-c122-430f-92d4-cc5fa9677693"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC4",
            name="Kumuliert",
            ahb_expression=None,
            section_name="Wertegranularität",
            index=269,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC4",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9605389c-ab85-49a3-89b2-06b6440a7184"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC5",
            name="1/4 Stunde",
            ahb_expression=None,
            section_name="Wertegranularität",
            index=270,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC5",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("29b65d74-59ac-492e-b403-e042f06408d7"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC7",
            name="höchste 1/4 Stunde im Monat",
            ahb_expression=None,
            section_name="Wertegranularität",
            index=271,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC7",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6f86b99b-6b29-4a4c-be88-1aed1c2bdeea"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zugeordnete Zählzeit",
            index=273,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("0d4d1ba2-8035-456f-a90e-68b14a1a0416"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zugeordnete Zählzeit",
            index=274,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("10d18f56-db2e-4813-82b1-138160a474a8"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z39",
            name="Code der Zählzeit",
            ahb_expression=None,
            section_name="Zugeordnete Zählzeit",
            index=275,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z39",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7b5650dc-445f-4a3e-9431-eb3b6fdc8482"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="0",
            name="Code der ZählzeitZ61",
            ahb_expression=None,
            section_name="Zugeordnete Zählzeit",
            index=276,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="0",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b3ed382c-7d40-4acb-89b1-a44565e482ec"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wert für Netzbetreiber relevant",
            index=278,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("430353e5-8118-4e8d-9d52-c7e52643d3fe"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wert für Netzbetreiber relevant",
            index=279,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("287dd1e5-edd1-4e51-a9ce-8c7372a5e17d"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZA8",
            name="NB",
            ahb_expression=None,
            section_name="Wert für Netzbetreiber relevant",
            index=280,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZA8",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("cd72f5c0-ffe5-4750-bbf9-e1de788b2e13"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=282,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5a5c397d-88ae-4b3a-a6e0-e8d0dda43fed"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=283,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("808cfb9f-bc4f-4f01-8a70-7334e8cb7ab0"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z84",
            name="Netznutzungsabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=284,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z84",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5353a6f0-68a4-468b-a4da-db3fdde89693"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z85",
            name="Bilanzkreisabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=285,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z85",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b9f91232-1714-450d-8a89-e063bafd085e"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z86",
            name="Mehrmindermengenabrec hnung\t[27P1..1]",
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=286,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z86",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6beeb0bb-2cad-402e-b82e-47ba173cdfa9"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z92",
            name="Übermittlung an das HKNR\t[24P1..1]",
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=287,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z92",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a65e57e8-e6df-4ac5-b05d-9bcdee6b12d9"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZD1",
            name="Blindstromabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=288,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZD1",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e6280367-82e8-4c45-98fd-ab3a914e9656"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZE1",
            name="Es liegt kein Verwendungszweck vor",
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=289,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZE1",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5eeee2ab-111e-4100-a141-812fbb245bcc"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wert für Lieferant relevant",
            index=291,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("c4779b52-b88a-4dce-a359-aadfbc01286f"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wert für Lieferant relevant",
            index=292,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2bb50604-0ed6-4c80-8f7b-7ff6d30a6137"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZA7",
            name="LF",
            ahb_expression=None,
            section_name="Wert für Lieferant relevant",
            index=293,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZA7",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("156e9e52-308c-4fd9-a72b-06f6fff0ff78"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=295,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("658695cc-0f17-4e11-bf2f-4525c6aa4096"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=296,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("17531a8f-4087-4e75-9712-72aff21b3fae"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z84",
            name="Netznutzungsabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=297,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z84",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0e0747c0-7c44-46dc-9c32-a60f7f03bd42"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z85",
            name="Bilanzkreisabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=298,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z85",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("63042356-1c3c-4536-a68f-d438d6c81c75"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z86",
            name="Mehrmindermengenabrec hnung\t[27P1..1]",
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=299,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z86",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0ef76dc3-ccc1-4503-92fc-08ad15f20aec"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z47",
            name="Endkundenabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=300,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z47",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("79d54581-219a-48d1-a12a-a4e56f51f1be"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZD1",
            name="Blindstromabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=301,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZD1",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ee9626ee-a8f4-4ed4-9f0c-d6b50402fdc9"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZE1",
            name="Es liegt kein Verwendungszweck vor",
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=302,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZE1",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("cd72ac13-6c58-4cd0-b054-38cb4ec70e5d"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wert für Übertragungsnetzbetreiber relevant",
            index=304,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("baed60fc-b1ef-4aa0-a955-5aaa93f9caa8"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wert für Übertragungsnetzbetreiber relevant",
            index=305,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2c064a56-63d5-459d-87a8-d6e67ebb98d7"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZA9",
            name="ÜNB",
            ahb_expression=None,
            section_name="Wert für Übertragungsnetzbetreiber relevant",
            index=306,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZA9",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2bd1c52c-7fa8-4cb2-b36f-cb4d824b47f1"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=308,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("37600b93-30c9-45d5-8299-569e7b1860d5"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=309,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e377784d-4e62-4571-9bfc-cbcfa7de78fa"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z85",
            name="Bilanzkreisabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=310,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z85",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("01f1de2b-b33e-456a-ad32-29c0b53e4bf9"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZB5",
            name="Zur Ermittlung der Ausgeglichenheit von Bilanzkreisen",
            ahb_expression=None,
            section_name="Verwendungszweck des Wertes",
            index=311,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZB5",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c77d26b3-7107-48bd-8be0-cd886cd668cc"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS-Daten der Marktlokation",
            index=313,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("0cafa1e3-d235-432b-87a5-eb60f54991d2"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS-Daten der Marktlokation",
            index=314,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5610a8aa-cc89-4485-9fd9-ba1c12fb3167"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z02",
            name="OBIS-Daten derMarktlokation",
            ahb_expression=None,
            section_name="OBIS-Daten der Marktlokation",
            index=315,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z02",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("57a93a64-929b-485d-a524-1e49f00b9ba7"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=317,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("64f88166-b691-457c-ae8c-88689ac9e5a3"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=318,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e6c7337a-77f8-481a-9c61-8baea3c12c9f"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z18",
            name="Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=319,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("fc8a5736-8d54-4090-bab9-82d56361f150"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID einer Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=320,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("77bff33d-c1bf-453b-82e1-97ed8b243bed"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS-Kennzahl der Marktlokation",
            index=322,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            ],
            segment_code="PIA",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b61e96ee-a7ce-498e-9f4b-afed8ee6b82a"),
            segment_group_key="SG8",
            segment_code="PIA",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS-Kennzahl der Marktlokation",
            index=323,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            ],
            segment_code="PIA",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("911ca3c5-f976-4ca7-8cee-cb99a03fb8a1"),
            segment_group_key="SG8",
            segment_code="PIA",
            data_element="4347",
            value_pool_entry="5",
            name="Produktidentifikation",
            ahb_expression=None,
            section_name="OBIS-Kennzahl der Marktlokation",
            index=324,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            ],
            segment_code="PIA",
            data_element_id="4347",
            qualifier="5",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("53227bcb-a56d-4b46-9058-a5e2b7d5f434"),
            segment_group_key="SG8",
            segment_code="PIA",
            data_element="7140",
            value_pool_entry=None,
            name="OBIS-Kennzahl der Marktlokation",
            ahb_expression=None,
            section_name="OBIS-Kennzahl der Marktlokation",
            index=325,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            ],
            segment_code="PIA",
            data_element_id="7140",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9c66f8aa-11ff-4d84-a5c3-5ba81c574acd"),
            segment_group_key="SG8",
            segment_code="PIA",
            data_element="7143",
            value_pool_entry="SRW",
            name="OBIS-Kennzahl",
            ahb_expression=None,
            section_name="OBIS-Kennzahl der Marktlokation",
            index=326,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            ],
            segment_code="PIA",
            data_element_id="7143",
            qualifier="SRW",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8ead52c9-5a20-413f-834e-e3d31aff2916"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zugeordnete Zählzeit",
            index=328,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("ef7d7bb9-77e3-48f6-b8ec-60d074a05a0b"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zugeordnete Zählzeit",
            index=329,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("942f61fc-79a7-4265-8551-fb44217454d8"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z39",
            name="Code der Zählzeit",
            ahb_expression=None,
            section_name="Zugeordnete Zählzeit",
            index=330,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z39",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("067025ce-fd20-4f93-8f0c-cf7194c35c30"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry=None,
            name="Code der Zählzeit",
            ahb_expression=None,
            section_name="Zugeordnete Zählzeit",
            index=331,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("450bdf3e-009d-48b9-9c58-8e7b39e548ff"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zugeordnete Zählzeitregister",
            index=333,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z38"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("25a8a796-f0c2-431b-b905-8275922dad4b"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zugeordnete Zählzeitregister",
            index=334,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z38"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("34d7d642-b400-4d55-85ac-dd8e9274a88e"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z38",
            name="Code desZählzeitregisters",
            ahb_expression=None,
            section_name="Zugeordnete Zählzeitregister",
            index=335,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z38"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z38",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("242fee12-a85c-44e6-b78c-476360ff99b4"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry=None,
            name="Code des Zählzeitregisters",
            ahb_expression=None,
            section_name="Zugeordnete Zählzeitregister",
            index=336,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z38"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("85f46860-6f82-4256-8e66-27713c49de85"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS Daten für Netzbetreiber relevant",
            index=338,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("c890b7f5-bfeb-470f-905a-5377a4869a02"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS Daten für Netzbetreiber relevant",
            index=339,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("17c5f699-3f60-46f3-a77f-202c849cd277"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZA8",
            name="NB",
            ahb_expression=None,
            section_name="OBIS Daten für Netzbetreiber relevant",
            index=340,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZA8",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("78ebf786-360e-4a76-9e4c-40ac1f5bf5fd"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=342,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d8df6a18-5d56-4757-b63d-11f614eee4ed"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=343,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4536ab50-f78f-4f2b-bbd2-bf4cf3fbd96b"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z84",
            name="Netznutzungsabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=344,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z84",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4cf5dc93-d7d7-404c-bf21-4aa2bb762871"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z85",
            name="Bilanzkreisabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=345,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z85",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("963525fd-a5bc-4b49-a858-48caa925cfc5"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z86",
            name="Mehrmindermengenabrec hnung\t[37P1..1]",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=346,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z86",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5a39d4e0-a991-462a-b3a4-0fd437ef69d8"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z92",
            name="Übermittlung an das HKNR\t[34P1..1]",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=347,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z92",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bd6c8f49-ec72-454a-9859-772a181a241d"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZD1",
            name="Blindstromabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=348,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZD1",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7e9d31f3-9b11-47fa-8fc2-043612ec8921"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZE1",
            name="Es liegt kein Verwendungszweck vor",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=349,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZE1",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("fa214db8-7592-475d-825e-33cce5129ba3"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS Daten für Marktrolle relevant",
            index=351,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("456641c1-1c7f-4152-b034-9a04975284f9"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS Daten für Marktrolle relevant",
            index=352,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a499a120-df24-4fd3-a364-4bf64260c3a8"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZA7",
            name="LF",
            ahb_expression=None,
            section_name="OBIS Daten für Marktrolle relevant",
            index=353,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZA7",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("984fe149-07ec-4924-b63c-f4d3718a0c68"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=355,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7dc64ec4-b498-4db1-a4fe-36549c0e11ee"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=356,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7710b471-9e06-421a-b30e-cf1d99cf2ec9"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=357,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("45023324-b89b-4087-84fe-f2fb3468e1a2"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z84",
            name="Netznutzungsabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=358,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z84",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("fc1d24ec-fc7e-48d6-9ebd-d8923c68eaf8"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z85",
            name="Bilanzkreisabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=359,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z85",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2415d189-4f48-403f-8b62-78cec7158327"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z86",
            name="Mehrmindermengenabrec hnung\t[37P1..1]",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=360,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z86",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("386f0f25-d3e8-43b6-8ce8-4d8edf0a51a2"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z47",
            name="Endkundenabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=361,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z47",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("94a691ab-3e86-4422-88d2-60d59258648f"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZD1",
            name="Blindstromabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=362,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZD1",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e128ebea-76cc-49ae-baf1-4bf2a2929d4d"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZE1",
            name="Es liegt kein Verwendungszweck vor",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=363,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZE1",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8e956cfc-7aa3-4f95-b77a-7529ebc09460"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS Daten für Übertragungsnetzbetreiber relevant",
            index=365,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("e9abfd1b-85ff-4f4b-8a89-e4fa63f4085b"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS Daten für Übertragungsnetzbetreiber relevant",
            index=366,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d444268b-0f2e-4d76-93df-28cb6261c547"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZA9",
            name="ÜNB",
            ahb_expression=None,
            section_name="OBIS Daten für Übertragungsnetzbetreiber relevant",
            index=367,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZA9",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2dadd5e6-2c69-4822-9105-54c0f04f62a9"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=369,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bb6fa356-e5a8-444e-8518-c0563c0b6973"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=370,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9dfd6ba7-396f-49d4-87bd-bd3b87473cca"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z85",
            name="Bilanzkreisabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=371,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z85",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2b9c6e4d-9bc9-4356-8d57-6c3361287706"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZB5",
            name="Zur Ermittlung der Ausgeglichenheit von Bilanzkreisen",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=372,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZB5",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("afdaf011-75c4-4fd5-8242-03e7d3a9813d"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Daten der Tranche",
            index=374,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("1667e2a1-7bf8-494b-b62b-cba4697abe3c"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Daten der Tranche",
            index=375,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9f1cac80-50f9-4160-8407-3a7340a12eb9"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z15",
            name="Daten der Tranche",
            ahb_expression=None,
            section_name="Daten der Tranche",
            index=376,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z15",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("318c8fda-86ac-402b-b27f-1070286c0926"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Tranche",
            index=378,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("20057b02-fb68-4991-9b9e-49229bad9b0d"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Tranche",
            index=379,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6d411bc6-3fbb-45db-9401-e609d8d2847f"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z20",
            name="Tranche",
            ahb_expression=None,
            section_name="Referenz auf die ID der Tranche",
            index=380,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z20",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("17a42d0d-920d-4cd2-85e4-cd696b49a733"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID der Tranche",
            ahb_expression=None,
            section_name="Referenz auf die ID der Tranche",
            index=381,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("db5397a6-e5aa-4590-9c6b-c054b1ebe6b2"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die Marktlokation der Tranche",
            index=383,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2025bd4d-bfcb-4ca9-98e6-dac07de6327b"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die Marktlokation der Tranche",
            index=384,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("005a66aa-0819-4ca0-96fc-b5a8e6cf0a92"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z16",
            name="zugeordneteMarktlokation",
            ahb_expression=None,
            section_name="Referenz auf die Marktlokation der Tranche",
            index=385,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z16",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("957b0a56-8931-4d89-99c8-aa5c3092655f"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID der Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die Marktlokation der Tranche",
            index=386,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ae5bc847-3cbe-4f88-9af3-3a4cccf47781"),
            segment_group_key="SG9",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Prozentualer Anteil der Tranche an der erzeugenden Marktlokation",
            index=388,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="11"),
            ],
            segment_code="QTY",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("6c711ee9-c71b-4b78-8e4b-c5d49d287a9b"),
            segment_group_key="SG9",
            segment_code="QTY",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Prozentualer Anteil der Tranche an der erzeugenden Marktlokation",
            index=389,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="11"),
            ],
            segment_code="QTY",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f77f01ac-7064-4153-9695-8084b1eab155"),
            segment_group_key="SG9",
            segment_code="QTY",
            data_element="6063",
            value_pool_entry="11",
            name="Aufteilungsmenge",
            ahb_expression=None,
            section_name="Prozentualer Anteil der Tranche an der erzeugenden Marktlokation",
            index=390,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="11"),
            ],
            segment_code="QTY",
            data_element_id="6063",
            qualifier="11",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("71d1a52c-d267-46e3-b854-a95602beccee"),
            segment_group_key="SG9",
            segment_code="QTY",
            data_element="6060",
            value_pool_entry=None,
            name="Mengenangabe in %",
            ahb_expression=None,
            section_name="Prozentualer Anteil der Tranche an der erzeugenden Marktlokation",
            index=391,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="11"),
            ],
            segment_code="QTY",
            data_element_id="6060",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f067e096-db05-47be-90e1-c8ec98765735"),
            segment_group_key="SG9",
            segment_code="QTY",
            data_element="6411",
            value_pool_entry="P1",
            name="Prozent",
            ahb_expression=None,
            section_name="Prozentualer Anteil der Tranche an der erzeugenden Marktlokation",
            index=392,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="11"),
            ],
            segment_code="QTY",
            data_element_id="6411",
            qualifier="P1",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f47b6e3e-9b0c-4bfa-a85d-db1e969d2067"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zugeordnete Marktpartner",
            index=394,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("e8a2a7e3-a6b1-440c-9389-3c3ae61954bc"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zugeordnete Marktpartner",
            index=395,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3b63c969-f7e4-471d-b6d4-2b5ffd48bb40"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZB3",
            name="ZugeordneterMarktpartner",
            ahb_expression=None,
            section_name="Zugeordnete Marktpartner",
            index=396,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZB3",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6bca3004-3921-435d-a339-bffe845ae08f"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Lieferant",
            index=398,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("50761d9b-166e-4839-9e5a-e868fd6c2a2a"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Lieferant",
            index=399,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("33818d33-2f2c-4d10-8266-2ce9f833a969"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z89",
            name="LF",
            ahb_expression=None,
            section_name="Lieferant",
            index=400,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z89",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("878089b6-789f-47ea-9b72-aa985fd393ff"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="1131",
            value_pool_entry=None,
            name="MP-ID",
            ahb_expression=None,
            section_name="Lieferant",
            index=401,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="1131",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1de7b76e-fb77-4532-ac21-f45eb92cedea"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Basis zur Bildung der Tranchengröße",
            index=403,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z37"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("321e26e4-dd53-4270-892a-e7528ab97b1b"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Basis zur Bildung der Tranchengröße",
            index=404,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z37"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("82cb1391-14cb-44cc-a5c9-94203631c427"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z37",
            name="Basis zur Bildung derTranchengröße",
            ahb_expression=None,
            section_name="Basis zur Bildung der Tranchengröße",
            index=405,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z37"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z37",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9a65032b-f0ab-4b36-9d40-f3bb8c76637b"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZC11",
            name="Prozentual",
            ahb_expression=None,
            section_name="Basis zur Bildung der Tranchengröße",
            index=406,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z37"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZC11",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("725f90a2-2002-450b-ba6c-abb1d4f11c1a"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZC12",
            name="Aufteilungsfaktor auf Basis von Referenzenträger/ installierter Leistung",
            ahb_expression=None,
            section_name="Basis zur Bildung der Tranchengröße",
            index=407,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z37"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZC12",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1dbeabd3-c002-4485-a56a-25ef795deb1b"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Erforderlicher Wert der Tranche",
            index=409,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("26078326-c9b5-4486-850d-89179ea1cf33"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Erforderlicher Wert der Tranche",
            index=410,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("60af72b6-5b02-478f-a8bd-90a407fb48b6"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z16",
            name="Erforderlicher Wert derTranche",
            ahb_expression=None,
            section_name="Erforderlicher Wert der Tranche",
            index=411,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z16",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("992748fb-2000-4d1a-8996-4d4d20278ee6"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Tranche",
            index=413,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("403e241b-c5b0-4c6a-a4c5-826f4310310c"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Tranche",
            index=414,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e44c532b-9465-472c-a88f-bbec89554ab5"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z20",
            name="Tranche",
            ahb_expression=None,
            section_name="Referenz auf die ID der Tranche",
            index=415,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z20",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f6664ce3-3561-461c-a25b-afe9e0aaace0"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID einer Tranche",
            ahb_expression=None,
            section_name="Referenz auf die ID der Tranche",
            index=416,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9a0fa441-4be9-4fef-8e54-0c6e996664b3"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Beschreibung erforderlicher Wert",
            index=418,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("f9dcee44-dbde-44a0-9b74-13c623331c32"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Beschreibung erforderlicher Wert",
            index=419,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("516373ba-98c3-4aef-8ca2-0be8532c9bd8"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z35",
            name="Beschreibungerforderlicher Wert",
            ahb_expression=None,
            section_name="Beschreibung erforderlicher Wert",
            index=420,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z35",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d87337d2-59b3-4115-a40a-93cd8f1cc965"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wert",
            index=422,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("baa630ee-fdc1-4b63-b682-f6a18a6d3c42"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wert",
            index=423,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4139bcce-8311-4d87-8a77-e280dcc1fd5e"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC2",
            name="Wirkarbeit",
            ahb_expression=None,
            section_name="Wert",
            index=424,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC2",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e963a3e3-e195-42d3-ba0a-3c28fe2c909f"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wertegranularität",
            index=426,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("de4d4613-5a9e-43ba-abe9-01d4f3b65aaa"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wertegranularität",
            index=427,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("50c46084-1382-45cb-af1e-93d402a324ab"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC5",
            name="1/4 Stunde",
            ahb_expression=None,
            section_name="Wertegranularität",
            index=428,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC5",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("406accb7-e3e7-435e-88cc-365c0726d861"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Werte für Netzbetreiber relevant",
            index=430,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("1a6fc9a0-ebf7-4b01-9f78-be6bf93070f4"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Werte für Netzbetreiber relevant",
            index=431,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3fe609fb-c673-46cd-96d5-b933b84fec6f"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZA8",
            name="NB",
            ahb_expression=None,
            section_name="Werte für Netzbetreiber relevant",
            index=432,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZA8",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("da2617d2-8522-4baf-a043-3b48000d999a"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=434,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("114b11a0-e5e2-4d33-87a6-6fce32f842f6"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=435,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e372f841-2c75-4e73-96cf-64b915350bd4"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z85",
            name="Bilanzkreisabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=436,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z85",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8f1dff24-e8af-46f1-9f6b-ffd740cc16a9"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z92",
            name="Übermittlung an das HKNR",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=437,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z92",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b4770cfd-bb7d-4bf3-8a60-9ebdb675cb4e"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS Daten für Marktrolle relevant",
            index=439,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("dab45efc-01f1-4e73-a8c3-1ce1edfebaae"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS Daten für Marktrolle relevant",
            index=440,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("96322f6b-62bd-47ae-9e3a-3144f5facec1"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZA7",
            name="LF",
            ahb_expression=None,
            section_name="OBIS Daten für Marktrolle relevant",
            index=441,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZA7",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("877e2734-4dbd-4ec5-921f-29d7343ff854"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=443,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ca989865-0e27-4d07-86f2-a33cf7be09d8"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=444,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b74699a3-cac3-46db-94de-c88dd809a96f"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z85",
            name="Bilanzkreisabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=445,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z85",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4ca391b3-85aa-4582-8fb4-20b468e4c38f"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z47",
            name="Endkundenabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=446,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z47",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("de045cc4-2bcf-43f8-978d-7a3511049341"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS Daten für Übertragungsnetzbetreiber relevant",
            index=448,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("116c8905-a612-4d7b-a982-089a664d752e"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS Daten für Übertragungsnetzbetreiber relevant",
            index=449,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6036cd1f-7c0f-409b-a6c3-b82b44220f12"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZA9",
            name="ÜNB",
            ahb_expression=None,
            section_name="OBIS Daten für Übertragungsnetzbetreiber relevant",
            index=450,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZA9",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("86e08bfb-76b5-4e7b-9f7c-57bb78a1a47e"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=452,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5b906ea2-fa32-4c1f-b25a-4f9420e7173b"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=453,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("34f3d7b7-e746-4f6d-ac67-aff54944dc08"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z85",
            name="Bilanzkreisabrechnung",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=454,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z85",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("df2aeed4-d813-404e-956d-d10662351529"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZB5",
            name="Zur Ermittlung der Ausgeglichenheit von Bilanzkreisen",
            ahb_expression=None,
            section_name="Verwendungszweck der Werte",
            index=455,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZB5",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ad5e5a9e-021a-4f3b-9a1a-e9578f704050"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Daten der Messlokation",
            index=457,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("2043913c-36ef-48bd-9511-85dea351f738"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Daten der Messlokation",
            index=458,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a6d7b2fb-c131-4fb1-8080-e1f143cd1d37"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z18",
            name="Daten der Messlokation",
            ahb_expression=None,
            section_name="Daten der Messlokation",
            index=459,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7899e2a0-4072-4e68-8f6b-d769da6326c3"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Messlokation",
            index=461,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("76818479-14f7-4097-b46e-6c3e3262fbad"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Messlokation",
            index=462,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("135a1c74-5e4e-4583-87c1-50ad7146ac2e"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z19",
            name="Messlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Messlokation",
            index=463,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z19",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("868eb3c6-d4fa-494a-976e-bcf7a53f350b"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID der Messlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Messlokation",
            index=464,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bece87e6-042c-404b-955f-c3376529e7bf"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die der Messlokation zugeordneten Marktlokation",
            index=466,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e1258a33-a15e-4e7a-b3ba-d3437ee7bb54"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die der Messlokation zugeordneten Marktlokation",
            index=467,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7dbd8993-819a-440f-a717-93bedb0153e7"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z16",
            name="zugeordneteMarktlokation",
            ahb_expression=None,
            section_name="Referenz auf die der Messlokation zugeordneten Marktlokation",
            index=469,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z16",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f644b16b-5c5e-41ff-8afe-b57f210c0dd6"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID der Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die der Messlokation zugeordneten Marktlokation",
            index=470,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("51905ff0-9aec-42e4-a185-018b69910eaf"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zugeordnete Marktpartner",
            index=472,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("86b74c54-0308-48de-aaba-28b7e3560365"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zugeordnete Marktpartner",
            index=473,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4427958c-9247-4d06-8af8-b5ade1ee78aa"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZB3",
            name="ZugeordneterMarktpartner",
            ahb_expression=None,
            section_name="Zugeordnete Marktpartner",
            index=474,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZB3",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("741f57c2-f2e5-4865-81c4-bd79215c419a"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Messstellenbetreiber",
            index=476,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e5310810-3e1c-40d7-acfa-136cfbc77a28"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Messstellenbetreiber",
            index=477,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3bf8e9d3-0228-4224-9b3e-0b29e66c4e2f"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z91",
            name="MSB",
            ahb_expression=None,
            section_name="Messstellenbetreiber",
            index=478,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z91",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0e5fe8e0-4a6d-483e-b5e6-e60851112772"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="1131",
            value_pool_entry=None,
            name="MP-ID",
            ahb_expression=None,
            section_name="Messstellenbetreiber",
            index=479,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="1131",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f863bd73-aaa0-44da-b200-37b914c927bb"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z19",
            name="Auf vertraglicher Grundlage gegenüber Anschlussnutzer / Anschlussnehmer",
            ahb_expression=None,
            section_name="Messstellenbetreiber",
            index=480,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z19",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("267629ee-082f-4637-9084-836a88ea9565"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z20",
            name="In der Ausübung der Weiterverpflichtung durch den gMSB",
            ahb_expression=None,
            section_name="Messstellenbetreiber",
            index=481,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z20",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e5fc63aa-5543-4b91-b557-9a0e37bbc1f3"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Messstellenbetreiber Alt",
            index=483,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bc350565-33b8-4e9b-b0ac-377200bfa122"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Messstellenbetreiber Alt",
            index=484,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a122125d-43ce-4001-89aa-1d7fb0c776de"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZB4",
            name="MSBA",
            ahb_expression=None,
            section_name="Messstellenbetreiber Alt",
            index=485,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZB4",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8def82c6-49c2-4068-bda3-fd16eb9da689"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="1131",
            value_pool_entry=None,
            name="MP-ID",
            ahb_expression=None,
            section_name="Messstellenbetreiber Alt",
            index=486,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
            ],
            segment_code="CAV",
            data_element_id="1131",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7701ab23-31cc-42ad-b51e-7a2e31f485b1"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Spannungsebene der Messlokation",
            index=488,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("c27f288e-ae68-4e83-b947-a63b9c71fe1b"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Spannungsebene der Messlokation",
            index=489,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("44f05e6a-6296-4c11-a9ea-ea532c2c6e2f"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="E04",
            name="Spannungsebene derMesslokation",
            ahb_expression=None,
            section_name="Spannungsebene der Messlokation",
            index=490,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="E04",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("16250a9f-0027-465e-b5b4-a2a5667f29b2"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Spannungsebene der Messlokation",
            index=492,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d7eab8d5-345a-42f9-8714-bc26af85bb35"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Spannungsebene der Messlokation",
            index=493,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7e3bd3f0-ae20-4484-8454-21d0d33575d4"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="E03",
            name="Höchstspannung",
            ahb_expression=None,
            section_name="Spannungsebene der Messlokation",
            index=494,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="E03",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1982258e-31b3-48c3-ba96-d65bca68236d"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="E04",
            name="Hochspannung",
            ahb_expression=None,
            section_name="Spannungsebene der Messlokation",
            index=495,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="E04",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("69365ce3-bdcc-4e93-93f1-0e3aa47005f4"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="E05",
            name="Mittelspannung",
            ahb_expression=None,
            section_name="Spannungsebene der Messlokation",
            index=496,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="E05",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7309f523-9010-4181-aedc-de01367bb894"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="E06",
            name="Niederspannung",
            ahb_expression=None,
            section_name="Spannungsebene der Messlokation",
            index=497,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="E06",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f6e38653-aabd-4642-ae0a-a512c0b8c057"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Gasqualität",
            index=499,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Y02"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("5d1956bd-ec85-497e-ba0a-cc54161571b8"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Gasqualität",
            index=500,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Y02"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9e5663ae-d8f3-41a2-8891-dc5cf115f30d"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Y02",
            name="Gasqualität",
            ahb_expression=None,
            section_name="Gasqualität",
            index=501,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Y02"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Y02",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6428c4ea-8252-4fd1-8fa3-acb3b39fd84d"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Gasqualität",
            index=503,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Y02"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("64c75f6e-79f7-4b66-9b73-6deae9a9068b"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Gasqualität",
            index=504,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Y02"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4058fbb7-3a5d-437b-b5e9-d436d3c02f4a"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Y04",
            name="H-Gas",
            ahb_expression=None,
            section_name="Gasqualität",
            index=505,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Y02"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Y04",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8dbd02a9-74de-482b-932e-1dc30354fda7"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Y05",
            name="L-Gas",
            ahb_expression=None,
            section_name="Gasqualität",
            index=506,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Y02"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Y05",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ac90e035-ac6b-45d0-92ca-6465dac45507"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungsumfang",
            index=508,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z01"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("2682966a-bb4f-4b30-aeb8-5a41e8a09fc6"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Verwendungsumfang",
            index=509,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z01"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("873f7fb1-1e17-4f90-9d00-0ab7af4618fd"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z01",
            name="Verwendungsumfang",
            ahb_expression=None,
            section_name="Verwendungsumfang",
            index=510,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z01"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z01",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("db4ddf9b-ca7f-4724-ae06-f07c295e8546"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z82",
            name="ID der prozessual behandelten Messlokation",
            ahb_expression=None,
            section_name="Verwendungsumfang",
            index=511,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z01"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z82",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e8e95ee4-a9cd-4e50-968b-036ee8f08e74"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z81",
            name="ID der Messlokation im Lokationsbündel",
            ahb_expression=None,
            section_name="Verwendungsumfang",
            index=512,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z01"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z81",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ddd48f09-421b-4dd8-aa77-4af6f19d42df"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Betriebszustand der Messlokation",
            index=514,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z32"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("9bb8b5c0-31e1-44d0-a63b-469d8d1557d3"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Betriebszustand der Messlokation",
            index=515,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z32"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7119d35a-46b8-4c9e-8769-a7a3c34741eb"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z32",
            name="Betriebszustand derMesslokation",
            ahb_expression=None,
            section_name="Betriebszustand der Messlokation",
            index=516,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z32"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z32",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c86c0641-6411-4132-808b-ce7dd47562a6"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZC1",
            name="Messlokation gesperrt und darf nicht entsperrt werden",
            ahb_expression=None,
            section_name="Betriebszustand der Messlokation",
            index=517,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z32"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZC1",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7dca2505-b111-4d8e-8d66-1ae4e31bd0a9"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZC2",
            name="Messlokation gesperrt und darf entsperrt werden",
            ahb_expression=None,
            section_name="Betriebszustand der Messlokation",
            index=518,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z32"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZC2",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("20962a3d-c0fd-496b-913e-16c6313442cf"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="ZC3",
            name="Messlokation im Regelbetrieb",
            ahb_expression=None,
            section_name="Betriebszustand der Messlokation",
            index=519,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z32"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="ZC3",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b7a8a765-3b42-459e-8ec3-f1513a415efd"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Erforderlicher Wert der Messlokation",
            index=521,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("4795eee5-3da7-4bff-bfd2-4e078729cd14"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Erforderlicher Wert der Messlokation",
            index=522,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7c704ce5-6e8f-45f8-b4f7-83c1c276ce18"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z19",
            name="Erforderlicher Wert derMesslokation",
            ahb_expression=None,
            section_name="Erforderlicher Wert der Messlokation",
            index=523,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z19",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("926e194e-8196-4b57-b0b0-240056f45143"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID einer Messlokation",
            index=525,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("71168e26-76e2-4be3-8afe-1887e20985e1"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID einer Messlokation",
            index=526,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("962344ba-df20-4911-b4b0-d8391a8ed40b"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z19",
            name="Messlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID einer Messlokation",
            index=527,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z19",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6b66b02d-ba95-4b3c-b882-94ab436be046"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID einer Messlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID einer Messlokation",
            index=528,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("82f27550-fcec-4270-8759-65c752476c51"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID einer Marktlokation",
            index=530,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("75ab2d43-7261-49e2-b50b-bccbed9c4ff8"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID einer Marktlokation",
            index=531,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("17ac7fe8-c25c-4864-9fb6-efb14ba83d9f"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z18",
            name="Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID einer Marktlokation",
            index=532,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b1a31e0a-ac02-4f55-b63a-1b285ea00ae0"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID einer Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID einer Marktlokation",
            index=533,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("13ee9675-fcf2-4698-95f2-100d714dab30"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Beschreibung erforderlicher Wert",
            index=535,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("903ff53e-d70a-471a-ae2e-582f1479983a"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Beschreibung erforderlicher Wert",
            index=536,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("baa9daef-0273-45cd-aae3-a2d5d71317b1"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z35",
            name="Beschreibungerforderlicher Wert",
            ahb_expression=None,
            section_name="Beschreibung erforderlicher Wert",
            index=537,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z35",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a89c13cb-4717-46b6-af6f-d3fff851d308"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wert",
            index=539,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2602daa7-faed-4311-af2d-27a9e2c37d92"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wert",
            index=540,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bc32cb5a-4b7f-4fc4-95eb-6d3bd2171c80"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC2",
            name="Wirkarbeit",
            ahb_expression=None,
            section_name="Wert",
            index=541,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC2",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("58ce68f5-458a-4cc8-8f01-082b941ca748"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC3",
            name="Blindarbeit",
            ahb_expression=None,
            section_name="Wert",
            index=542,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC3",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1a24b223-04d5-47dc-b5de-a1da4bb8d799"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wertegranularität",
            index=544,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("22825aa3-1806-4ead-b49d-be1d9d9bd2a3"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wertegranularität",
            index=545,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("92e31b19-c14e-4bc6-8bec-d0b84fea6122"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC4",
            name="Kumuliert",
            ahb_expression=None,
            section_name="Wertegranularität",
            index=546,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC4",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("615c43e0-b38c-4c4a-9326-b2e8282e8c94"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC5",
            name="1/4 Stunde",
            ahb_expression=None,
            section_name="Wertegranularität",
            index=547,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC5",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d13f0ff6-79c2-4d3f-bd5c-d32444bc339e"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC6",
            name="Eine Stunde",
            ahb_expression=None,
            section_name="Wertegranularität",
            index=548,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC6",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b53e52f8-17b1-4d98-83d5-f492c77bc9d8"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC7",
            name="höchste 1/4 Stunde im Monat",
            ahb_expression=None,
            section_name="Wertegranularität",
            index=549,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC7",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8101d0c7-53f2-4aa6-8919-4b3d327330e4"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Energieflussrichtung",
            index=551,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6843d619-14a5-4627-ad07-7316d9a17a54"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Energieflussrichtung",
            index=552,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7c31c617-92a6-4d3a-882f-c828ec05fc89"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z71",
            name="Verbrauch",
            ahb_expression=None,
            section_name="Energieflussrichtung",
            index=553,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z71",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("afac8497-8e3d-4841-a872-894d1a8df334"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z72",
            name="Erzeugung",
            ahb_expression=None,
            section_name="Energieflussrichtung",
            index=554,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z72",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("52b877b2-b669-4a27-b6f7-4990a9106681"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Notwendigkeit einer zweiten Messung",
            index=556,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4d51f9c4-7869-47e5-8273-e76df2d30b1f"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Notwendigkeit einer zweiten Messung",
            index=557,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1489cf44-27e7-4403-a4a2-ec30ae2f2bce"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC9",
            name="Zweite Messung fürVergleichsmessungerforderlich",
            ahb_expression=None,
            section_name="Notwendigkeit einer zweiten Messung",
            index=558,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC9",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2b21aec9-118e-4c2b-8c77-57ff8cf718ee"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z06",
            name="vorhanden",
            ahb_expression=None,
            section_name="Notwendigkeit einer zweiten Messung",
            index=559,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z06",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9c51c13e-c0b9-43a2-9796-799117efd413"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z07",
            name="nicht vorhanden",
            ahb_expression=None,
            section_name="Notwendigkeit einer zweiten Messung",
            index=560,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z07",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5c9fe01a-b41a-4784-b9f7-68e826260e1b"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Werteübermittlung an den NB aufgrund weiterem Verwendungszweck",
            index=562,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1a51cd04-4a6d-4144-9816-2ca272fd0741"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Werteübermittlung an den NB aufgrund weiterem Verwendungszweck",
            index=563,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("477c3878-5133-4e24-864e-8b19ca08f25a"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z88",
            name="NB",
            ahb_expression=None,
            section_name="Werteübermittlung an den NB aufgrund weiterem Verwendungszweck",
            index=564,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z88",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("58139f15-1223-4aea-b6fe-707261065424"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z06",
            name="vorhanden",
            ahb_expression=None,
            section_name="Werteübermittlung an den NB aufgrund weiterem Verwendungszweck",
            index=565,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z06",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f24d3238-63cb-4690-b655-ccd970e4779e"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z07",
            name="nicht vorhanden",
            ahb_expression=None,
            section_name="Werteübermittlung an den NB aufgrund weiterem Verwendungszweck",
            index=566,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z07",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("088d2b35-2a58-4cfe-abf5-f175a8479ee7"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zugeordnete Zählzeit",
            index=568,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("be0b6a82-fb2b-4f6d-aecd-45ffebdc1f69"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zugeordnete Zählzeit",
            index=569,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c0c7b40b-d8f4-49c5-8d66-2e962c3cbd88"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z39",
            name="Code der Zählzeit",
            ahb_expression=None,
            section_name="Zugeordnete Zählzeit",
            index=570,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z39",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9c1d3d4c-c6b1-4a80-998c-a8e660770035"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="0",
            name="Code der ZählzeitZ61",
            ahb_expression=None,
            section_name="Zugeordnete Zählzeit",
            index=571,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="0",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1936c919-2cbb-4447-9844-702baaf7dfc2"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss [78] u[138]",
            section_name="Zähleinrichtungsdaten",
            index=573,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("c253792e-6287-4e0c-8d99-437f4264c769"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Zähleinrichtungsdaten",
            index=574,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8aef529a-8e95-475e-bb8a-b560cf06a0e8"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z03",
            name="Zähleinrichtungsdaten",
            ahb_expression="X",
            section_name="Zähleinrichtungsdaten",
            index=575,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z03",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bedafa17-86d9-4d47-9cb8-3660667b5d65"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID einer Messlokation",
            index=577,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a4e41c77-c822-49e2-809d-434a535d4a8f"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID einer Messlokation",
            index=578,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("28a64973-68b9-4c46-a7e3-a05971c81263"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z19",
            name="Messlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID einer Messlokation",
            index=579,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z19",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("cba86052-07f7-49b0-bd12-dc7cf3ea3d25"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID einer Messlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID einer Messlokation",
            index=580,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4137e381-0938-466d-95c7-eeb106f258d4"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf das Smartmeter-Gateway",
            index=582,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0dd0775f-5ce4-41cd-8e15-8ea83819b1db"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf das Smartmeter-Gateway",
            index=583,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d5aa0a62-99e7-4977-afe0-f6ea11c42310"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z14",
            name="Smartmeter-Gateway",
            ahb_expression=None,
            section_name="Referenz auf das Smartmeter-Gateway",
            index=584,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z14",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0ffb7a7b-642a-454d-8889-035a1669a9a9"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="Identifikation / Nummer desSmartmeter-Gateways",
            ahb_expression=None,
            section_name="Referenz auf das Smartmeter-Gateway",
            index=585,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("28afdea4-24df-447e-bf3d-757f314bfe11"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Zählertyp",
            index=587,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("b4db2d2d-f92c-4c48-8022-415fcf147865"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Zählertyp",
            index=588,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("cbd90601-86a6-463d-9c7d-7b560d2cb1d9"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="E13",
            name="Zählertyp",
            ahb_expression="X",
            section_name="Zählertyp",
            index=589,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="E13",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4bd10987-8894-4507-9fff-29473cf4d854"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zählertyp",
            index=591,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c6565fdf-0183-4916-88eb-af2407cfefbc"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zählertyp",
            index=592,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bafcca92-d83f-4ea9-87b3-0e84ce3b2316"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="AHZ",
            name="analoger Haushaltszähler (Drehstrom)",
            ahb_expression=None,
            section_name="Zählertyp",
            index=593,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="AHZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("66f0c627-24a2-4cf2-a0a7-51d0683d4147"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="WSZ",
            name="analoger Wechselstromzähler",
            ahb_expression=None,
            section_name="Zählertyp",
            index=594,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="WSZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1ba326b0-18c1-450f-90cc-09c5cf18b45c"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="LAZ",
            name="Lastgangzähler",
            ahb_expression=None,
            section_name="Zählertyp",
            index=595,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="LAZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2d29b203-d17a-413d-82b0-5592cdc8cacf"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="MAZ",
            name="Maximumzähler",
            ahb_expression=None,
            section_name="Zählertyp",
            index=596,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="MAZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("fa1268a3-288b-476b-a113-3ebd60f944bb"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="MME",
            name="moderne Messeinrichtung nach MsbG",
            ahb_expression=None,
            section_name="Zählertyp",
            index=597,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="MME",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e68cae68-359d-4fb8-8c79-5fc3c9c0eed1"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="DKZ",
            name="Drehkolbengaszähler",
            ahb_expression=None,
            section_name="Zählertyp",
            index=598,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="DKZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("dce35c37-5429-4c7c-af8c-1225cc529fb4"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="BGZ",
            name="Balgengaszähler",
            ahb_expression=None,
            section_name="Zählertyp",
            index=599,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="BGZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("da1115a6-f4a9-4159-9889-f0354298460c"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="TRZ",
            name="Turbinenradgaszähler",
            ahb_expression=None,
            section_name="Zählertyp",
            index=600,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="TRZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1a822a13-1575-4eb8-beda-13efca19d16c"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="UGZ",
            name="Ultraschallgaszähler",
            ahb_expression=None,
            section_name="Zählertyp",
            index=601,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="UGZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9e891816-e08d-4f3c-851c-17844d78b3c6"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="WGZ",
            name="Wirbelgaszähler",
            ahb_expression=None,
            section_name="Zählertyp",
            index=602,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="WGZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0a505e47-7105-499f-8263-005d0b127ec0"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="MRG",
            name="Messdatenregistriergerät",
            ahb_expression=None,
            section_name="Zählertyp",
            index=603,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="MRG",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("27d553ff-e1cb-4466-bbcf-0a448a1ebf00"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="EHZ",
            name="elektronischer Haushaltszähler",
            ahb_expression=None,
            section_name="Zählertyp",
            index=604,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="EHZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2ab87fa3-4a65-423f-93d5-0f8d2206778d"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="IVA",
            name="Individuelle Abstimmung (Sonderausstattung)",
            ahb_expression=None,
            section_name="Zählertyp",
            index=605,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="IVA",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d9ff8b3c-a945-416e-aa95-3bebac1343ee"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z01",
            name="EDL40",
            ahb_expression=None,
            section_name="Zählertyp",
            index=606,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z01",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7da76915-7a00-4322-80a1-07cd294a58ea"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z02",
            name="EDL21",
            ahb_expression=None,
            section_name="Zählertyp",
            index=607,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z02",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("397a30e0-a85d-4de0-a7fa-bb89d048a135"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z03",
            name="sonstiger EHZ",
            ahb_expression=None,
            section_name="Zählertyp",
            index=608,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z03",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b1f82d55-8727-4da4-9f2e-f66fe1f51e29"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z04",
            name="Standard",
            ahb_expression=None,
            section_name="Zählertyp",
            index=609,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z04",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("cbeee636-b573-4956-ae0f-7ed124d4cbe3"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z05",
            name="MeDa-Zähler",
            ahb_expression=None,
            section_name="Zählertyp",
            index=610,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z05",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0bb4f329-5160-41ba-b1ac-e4912f9768be"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=612,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("38e6f799-4bfe-4ca3-be41-6e574128c3e9"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=613,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2a9c7929-450a-4edc-9326-2c362fd2e5ae"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G10",
            name="Gaszähler G10",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=614,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G10",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c78ab316-e728-46cc-a6dd-7536a8190182"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G100",
            name="Gaszähler G100",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=615,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G100",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5938e5d0-1124-425f-b23d-abf8a52f38ad"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G1000",
            name="Gaszähler G1000",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=616,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G1000",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("abda186b-d4e7-457e-8980-447e16447fe2"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G10000",
            name="Gaszähler G10000",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=617,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G10000",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("21bd8264-e5ed-40ea-9eb8-0b73d3c5a80d"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G12500",
            name="Gaszähler G12500",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=618,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G12500",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("955059d9-bb11-4f5e-842b-77ff96e90006"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G16",
            name="Gaszähler G16",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=619,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G16",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("81966a0f-7624-4854-b1a8-4cd38466f03a"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G160",
            name="Gaszähler G160",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=620,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G160",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("39c8f538-0b46-41d4-9804-35e89b69ad86"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G1600",
            name="Gaszähler G1600",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=621,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G1600",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9e5f3d07-809e-4c27-bcbe-5e0933c69e04"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G16000",
            name="Gaszähler G16000",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=622,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G16000",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7475c0af-88e6-4d96-8d95-b9c86b873d76"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Gaszähler G2.5",
            name="G2.5",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=623,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Gaszähler G2.5",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2390df70-2adf-41c8-acbd-62946d191d91"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G25",
            name="Gaszähler G25",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=624,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G25",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2ab3e5b7-62bc-4c36-8b85-782f4a76e38f"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G250",
            name="Gaszähler G250",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=625,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G250",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("56314591-9516-4ebc-9d6f-e3e2e7e2b233"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G2500",
            name="Gaszähler G2500",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=626,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G2500",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("dd78fca8-064d-4950-bdd1-9612cce70967"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G350",
            name="Gaszähler G350",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=627,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G350",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9cbb2baa-eff0-4574-addf-b78d20515706"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G4",
            name="Gaszähler G4",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=628,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G4",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6598a496-1620-4d57-9621-9b8957f617d2"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G40",
            name="Gaszähler G40",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=629,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G40",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("351ca306-d951-480a-b009-aca7f2f6e39c"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G400",
            name="Gaszähler G400",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=630,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G400",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("02b26f35-e33b-4c22-a5b5-7d09e280464e"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G4000",
            name="Gaszähler G4000",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=631,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G4000",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("dd470333-5e9c-4160-9acb-74130fe7ac3f"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G6",
            name="Gaszähler G6",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=632,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G6",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e645a75c-1bde-4c9c-9b5a-9226fb1b5be9"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G65",
            name="Gaszähler G65",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=633,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G65",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2b267536-39fc-4e82-b711-fa203b4f5499"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G650",
            name="Gaszähler G650",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=634,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G650",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2a7541bf-4764-4365-be06-451a28fcf860"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="G6500",
            name="Gaszähler G6500",
            ahb_expression=None,
            section_name="Zählergröße (Gas)",
            index=635,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="G6500",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bd2e8ded-b54c-410c-a085-41e627cb2885"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=637,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6c3cb54f-0b47-4f37-9fc5-db70964a7344"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Identifikation / Nummer des Gerätes",
            index=638,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7f4d0a25-ba0d-4684-ae73-16060f235feb"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z30",
            name="Gerätenummer",
            ahb_expression="X",
            section_name="Identifikation / Nummer des Gerätes",
            index=639,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z30",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c1b6661b-ab2c-439e-afc0-ff2b37619869"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry=None,
            name="Gerätenummer",
            ahb_expression="X",
            section_name="Identifikation / Nummer des Gerätes",
            index=640,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("927291ed-12b5-444c-912d-95fcb1bd44be"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Registeranzahl",
            index=642,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9d06c3ea-97f2-464b-8252-3eb50d3547cd"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Registeranzahl",
            index=643,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8803c7d7-77d1-4ca4-ad14-4fb5eb464621"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ETZ",
            name="Eintarif",
            ahb_expression=None,
            section_name="Registeranzahl",
            index=644,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ETZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3007ecd3-0bef-4d9f-8222-6c31bb9cf842"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZTZ",
            name="Zweitarif",
            ahb_expression=None,
            section_name="Registeranzahl",
            index=645,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZTZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6c7f8844-425e-42c2-829e-7dfb9856522d"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="NTZ",
            name="Mehrtarif",
            ahb_expression=None,
            section_name="Registeranzahl",
            index=646,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="NTZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d6d2b4b6-f908-4ee4-b736-61f8b573d932"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Energierichtung",
            index=648,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6ccf9fd6-7888-488a-912d-216c69c4eae9"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Energierichtung",
            index=649,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bb28adb5-9ee9-4faa-b1f5-d4074340acf8"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ERZ",
            name="Einrichtungszähler",
            ahb_expression=None,
            section_name="Energierichtung",
            index=650,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ERZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a42a17d8-c48e-463c-a63f-8cde02374d5f"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZRZ",
            name="Zweirichtungszähler",
            ahb_expression=None,
            section_name="Energierichtung",
            index=651,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZRZ",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f7e4d283-4809-43cf-8dbf-7344cdc77606"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Fernschaltung der Zähleinrichtung",
            index=653,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a7af529b-a550-4005-b8e8-bc3f56db5fda"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Fernschaltung der Zähleinrichtung",
            index=654,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("eb869dd9-74dd-471d-8870-8cd1beb54247"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z58",
            name="Fernschaltung derZähleinrichtung",
            ahb_expression=None,
            section_name="Fernschaltung der Zähleinrichtung",
            index=655,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z58",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("148c7eea-c322-4b44-b725-a44d849a94b9"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z06",
            name="vorhanden",
            ahb_expression=None,
            section_name="Fernschaltung der Zähleinrichtung",
            index=656,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z06",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7fcc30b0-86c9-43b4-b1fa-e5cde8341199"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z07",
            name="nicht vorhanden",
            ahb_expression=None,
            section_name="Fernschaltung der Zähleinrichtung",
            index=657,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z07",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5a871549-658e-461f-af79-fc3f1a3bb14c"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Art der Volumenerfassung (Gas)",
            index=659,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3f6b0911-4635-4b49-8213-cb7687a90619"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Art der Volumenerfassung (Gas)",
            index=660,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("26037989-360d-4075-8420-b1641f686e12"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC0",
            name="Art derVolumenerfassung",
            ahb_expression=None,
            section_name="Art der Volumenerfassung (Gas)",
            index=661,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC0",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e7bf4ed2-8a68-4640-90a4-c0e698c5dccd"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z16",
            name="Hochfrequenzsonde",
            ahb_expression=None,
            section_name="Art der Volumenerfassung (Gas)",
            index=662,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z16",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("81ead848-7347-40bf-9e6a-af2ff852f4d0"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Befestigungsart Zähleinrichtung",
            index=664,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("c156ba6f-45c8-4f40-8f5d-a64076f7e8e1"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Befestigungsart Zähleinrichtung",
            index=665,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1fd11dce-44ab-437c-9ba7-7a2fba24df51"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z28",
            name="BefestigungsartZähleinrichtung",
            ahb_expression=None,
            section_name="Befestigungsart Zähleinrichtung",
            index=666,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z28",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e05ca987-9a3f-4f7d-ba15-15680bdc8d7d"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Befestigungsart Zähleinrichtung",
            index=668,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3c1d523a-b6f8-4bf7-9a0c-14257572839e"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Befestigungsart Zähleinrichtung",
            index=669,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5a97baa1-98c0-4c17-ba3a-5fd89f495fc5"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="BKE",
            name="Stecktechnik (Befestigungs- und Kontaktierungseinrichtun g)",
            ahb_expression=None,
            section_name="Befestigungsart Zähleinrichtung",
            index=670,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="BKE",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ea973d00-48e8-40b1-bbd1-1111593ddff9"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="DPA",
            name="3-Dreipunktaufhängung",
            ahb_expression=None,
            section_name="Befestigungsart Zähleinrichtung",
            index=671,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="DPA",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5f0330f3-00bb-452c-93c4-3f65185d7ff3"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="HUT",
            name="Hutschiene",
            ahb_expression=None,
            section_name="Befestigungsart Zähleinrichtung",
            index=672,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="HUT",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("42317192-c1e6-4614-82af-415a471a9212"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z31",
            name="Einstutzen-Zähler",
            ahb_expression=None,
            section_name="Befestigungsart Zähleinrichtung",
            index=673,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z31",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("07cd981b-7856-41d4-ae0b-6bb4c1f34bf8"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z32",
            name="Zweistutzen-Zähler",
            ahb_expression=None,
            section_name="Befestigungsart Zähleinrichtung",
            index=674,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z32",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ddaac857-44d2-4f0f-b1c4-ba7809349425"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Messwerterfassung an der Zähleinrichtung",
            index=676,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E12"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("d6785c5d-3715-41ed-930e-05de90198eac"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Messwerterfassung an der Zähleinrichtung",
            index=677,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E12"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f2cf58a2-8c18-4658-b39d-a7d427ea4bc3"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="E12",
            name="Messwerterfassung ander Zähleinrichtung",
            ahb_expression=None,
            section_name="Messwerterfassung an der Zähleinrichtung",
            index=678,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E12"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="E12",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b60c4794-949d-4fa7-83e0-572507bf31ad"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Messwerterfassung an der Zähleinrichtung",
            index=680,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E12"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("befe952e-1a72-4dcd-82ec-6292bffc7933"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Messwerterfassung an der Zähleinrichtung",
            index=681,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E12"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5fd1a466-fd42-4456-8909-eda60aa1220e"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="AMR",
            name="fernauslesbare Zähler",
            ahb_expression=None,
            section_name="Messwerterfassung an der Zähleinrichtung",
            index=682,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E12"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="AMR",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("60f9c3d5-7f33-4b08-99aa-67ac4ed1ac10"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="MMR",
            name="manuell ausgelesene Zähler",
            ahb_expression=None,
            section_name="Messwerterfassung an der Zähleinrichtung",
            index=683,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E12"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="MMR",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0623ec63-9f23-456f-bd08-f43272494b1b"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Mengenumwerterdaten",
            index=685,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("d8452d11-a645-4af0-bcbe-5e94a51b767b"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Mengenumwerterdaten",
            index=686,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ac452b1b-8297-4c9e-98a0-e536e2d791dd"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z09",
            name="Mengenumwerter-Daten",
            ahb_expression=None,
            section_name="Mengenumwerterdaten",
            index=687,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z09",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("31fdee82-77ac-4698-8a0e-59923233dff9"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=689,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0d9264c8-e530-45d0-9011-e24ada0895f7"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=690,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("20a63fd8-27b4-49b8-9b9d-41a61e5be561"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="MG",
            name="Gerätenummer desZählers",
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=691,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="MG",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("22089704-5911-4141-b5d0-e3c64de9030b"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=692,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("637c3c50-22a7-4354-8637-2493d5490722"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Mengenumwerter",
            index=694,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("cfe7ea0e-4c99-4d31-9fee-09d70a708ba4"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Mengenumwerter",
            index=695,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("33d7854d-67da-47c4-b61a-046b735178d4"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z64",
            name="Mengenumwerter",
            ahb_expression=None,
            section_name="Mengenumwerter",
            index=696,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z64",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8d9b29f6-99a7-46dd-b3c6-ca685f4a45a9"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=698,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("64f5d771-09ee-44dd-9b31-5f1932e685a4"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=699,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bad3cd4c-2166-4cb6-a051-9ca2d829a25c"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z30",
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=700,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z30",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4bf5a04d-144f-484c-a930-548c50cfd979"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry=None,
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=701,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5d4facbf-cdfd-4f24-a0fa-fe4bc7046505"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Mengenumwertertyp",
            index=703,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("58268d41-09fe-4bba-8fe4-8e9db65f0d92"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Mengenumwertertyp",
            index=704,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ff8fdd85-8b74-4a67-aec8-dc09cac35553"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="DMU",
            name="Dichtemengenumwerter",
            ahb_expression=None,
            section_name="Mengenumwertertyp",
            index=705,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="DMU",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e5586dd0-8dec-41b5-84e6-f87e2b393be4"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="TMU",
            name="Temperaturmengenumwe rter",
            ahb_expression=None,
            section_name="Mengenumwertertyp",
            index=706,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="TMU",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0c23dd4b-d155-4bfa-b64f-b3cf09d1de06"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZMU",
            name="Zustandsmengenumwert er",
            ahb_expression=None,
            section_name="Mengenumwertertyp",
            index=707,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZMU",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("21144f63-ac60-4b5c-bb16-d350636ade15"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Art der Volumenerfassung (Gas)",
            index=709,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e414008b-4056-4414-a0b5-1766d668601f"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Art der Volumenerfassung (Gas)",
            index=710,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("14f987b4-2be5-4431-b93a-f5158fbc1390"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ZC0",
            name="Art derVolumenerfassung",
            ahb_expression=None,
            section_name="Art der Volumenerfassung (Gas)",
            index=711,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ZC0",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f800eda6-fe37-4261-9e5e-0588ed7930c5"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z17",
            name="Kennlinienkorrektur",
            ahb_expression=None,
            section_name="Art der Volumenerfassung (Gas)",
            index=712,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z17",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b3a5cdb5-bbf9-4312-bd66-27704cf4f6bc"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry="Z18",
            name="Schleichmengenunterdrü ckung",
            ahb_expression=None,
            section_name="Art der Volumenerfassung (Gas)",
            index=713,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("56bbf06b-5e18-4183-8147-58d37f61902d"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS-Daten der Zähleinrichtung / Mengenumwerter",
            index=715,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("f2bb40e6-05d4-4146-8136-08747ee79057"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS-Daten der Zähleinrichtung / Mengenumwerter",
            index=716,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9bb7ecf6-539c-4754-b0f8-c538e34ebdb4"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z20",
            name="OBIS-Daten derZähleinrichtung",
            ahb_expression=None,
            section_name="OBIS-Daten der Zähleinrichtung / Mengenumwerter",
            index=717,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z20",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("da6bc427-f269-46b4-8458-266359e02b3f"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=719,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ac28b16a-a51a-4bec-b278-691043ba9cca"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=720,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("cf0e7f4f-1b6b-4ea3-a2ef-13a9c12e6f09"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="MG",
            name="Gerätenummer des Zählers",
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=721,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="MG",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("eb0bf1e0-4f1f-4328-aca1-68f485b69961"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z11",
            name="Gerätenummer des Mengenumwerters",
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=722,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z11",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("808c4699-0da1-41e1-8e98-94bddc629394"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z14",
            name="Smartmeter-Gateway",
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=723,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z14",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("79e8c765-534a-48ec-b572-a0cd50eefaa2"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=724,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ca3c07c8-ca9d-4d8f-b5b0-dd7eb6142793"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID einer Marktlokation",
            index=726,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d20faecb-1378-4780-8123-4fb9afd00ca1"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID einer Marktlokation",
            index=727,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c63ced3c-0b05-45b6-9594-5175cd140fb1"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z18",
            name="Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID einer Marktlokation",
            index=728,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ba2b5bd5-1232-416a-8b99-4642228426de"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID einer Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID einer Marktlokation",
            index=729,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5e2672d5-5f3c-4986-803b-1db3c25273ab"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Konfigurnations-ID",
            index=731,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("cfa34b52-9673-40db-80ef-a230d3fbe84d"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Konfigurnations-ID",
            index=732,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("63e5e6a1-304d-4f21-90c4-5f500f137de1"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="AGK",
            name="Anwendungsreferenznummer",
            ahb_expression=None,
            section_name="Konfigurnations-ID",
            index=734,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="AGK",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("60cb54de-43bd-4ca3-a920-9356675b897e"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="Referenz, Identifikation",
            ahb_expression=None,
            section_name="Konfigurnations-ID",
            index=735,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2b6c8c6b-b6d3-4bde-89bb-0e6034d4e37f"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS-Kennzahl der Zähleinrichtung / Mengenumwerter",
            index=737,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="PIA",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3a280f61-79db-4144-841d-16b6145c753b"),
            segment_group_key="SG8",
            segment_code="PIA",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="OBIS-Kennzahl der Zähleinrichtung / Mengenumwerter",
            index=738,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="PIA",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("06234391-2e3f-460d-bfc3-1cb29e346efa"),
            segment_group_key="SG8",
            segment_code="PIA",
            data_element="4347",
            value_pool_entry="5",
            name="Produktidentifikation",
            ahb_expression=None,
            section_name="OBIS-Kennzahl der Zähleinrichtung / Mengenumwerter",
            index=739,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="PIA",
            data_element_id="4347",
            qualifier="5",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("36b04bab-1a94-4a87-afb0-8414aca8a25a"),
            segment_group_key="SG8",
            segment_code="PIA",
            data_element="7140",
            value_pool_entry=None,
            name="OBIS-Kennzahl derZähleinrichtung",
            ahb_expression=None,
            section_name="OBIS-Kennzahl der Zähleinrichtung / Mengenumwerter",
            index=740,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="PIA",
            data_element_id="7140",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e2bbbd72-4edd-4e54-b72d-ba1cd2ffdf49"),
            segment_group_key="SG8",
            segment_code="PIA",
            data_element="7143",
            value_pool_entry="SRW",
            name="OBIS-Kennzahl",
            ahb_expression=None,
            section_name="OBIS-Kennzahl der Zähleinrichtung / Mengenumwerter",
            index=741,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            ],
            segment_code="PIA",
            data_element_id="7143",
            qualifier="SRW",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("be5f8dd8-f23e-4d76-85b7-4a8aa649f423"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Vor- und Nachkommastellen des Zählwerks",
            index=743,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("d3c5c965-bef0-4ed4-9afa-1e38d4bb7137"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Vor- und Nachkommastellen des Zählwerks",
            index=745,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9b357235-2ecf-4ed5-b5af-07604a33cdd2"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="11",
            name="Produkt",
            ahb_expression=None,
            section_name="Vor- und Nachkommastellen des Zählwerks",
            index=746,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="11",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f617d34c-eea2-450c-bb36-3d98aa8f540e"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z33",
            name="Vor- undNachkommastellen desZählwerkes",
            ahb_expression=None,
            section_name="Vor- und Nachkommastellen des Zählwerks",
            index=747,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z33",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4c5dfcee-36f5-4769-bde2-ded1eaf080c6"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Vor- und Nachkommastellen des Zählwerks",
            index=749,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f5e8c992-28da-4d41-9c22-fdf93dd7e0b7"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Vor- und Nachkommastellen des Zählwerks",
            index=750,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("29705a83-643c-491f-9189-a0c91e6e89fc"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry=None,
            name="Angabe der Vorkommastelle",
            ahb_expression=None,
            section_name="Vor- und Nachkommastellen des Zählwerks",
            index=751,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1ff82b6f-61d9-489b-b0c9-5d5e698e58c2"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry=None,
            name="Angabe der Nachkommastelle",
            ahb_expression=None,
            section_name="Vor- und Nachkommastellen des Zählwerks",
            index=752,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a295be95-df4c-49bf-ab34-984a23b7cd1d"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wandlerdaten",
            index=754,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("f88843c2-e755-41df-9e13-a021bb319e52"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wandlerdaten",
            index=755,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c65a9959-656a-4a01-abf4-12c42a00b707"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z04",
            name="Wandler-Daten",
            ahb_expression=None,
            section_name="Wandlerdaten",
            index=756,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z04",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bf337621-ab67-4575-aa5d-f16920f54279"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=758,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bbeec90a-e06e-4101-b6ff-4c85468993a4"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=759,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("319bd81c-215a-477d-8f8a-505fc68b7dce"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="MG",
            name="Gerätenummer desZählers",
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=760,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="MG",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e2288c1a-12a3-453c-b3bf-43233b19de5c"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=761,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("22a39cdc-b3f8-4eab-97fb-0240a5b765db"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wandler",
            index=763,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("444b0b16-4601-4b97-a024-c45d603053cc"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wandler",
            index=764,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("170de93c-d7ec-49fe-9f19-25056e68826f"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z25",
            name="Wandler",
            ahb_expression=None,
            section_name="Wandler",
            index=765,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z25",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bb381832-a59a-428c-a0e9-d5ae83812b31"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=767,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("717e6f95-7034-4e4a-9669-c534d236a140"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=768,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2186cecb-55dd-4266-b27b-58bec1e7cbd6"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z30",
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=770,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z30",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("48d2b811-8cbe-4796-8302-de0e3c311d65"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry=None,
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=771,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3db87c69-7b2f-401c-8e6c-9d5eb801f6ac"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wandlertyp und Faktor",
            index=773,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3dec70fb-f495-457f-a57c-4a399ca73160"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Wandlertyp und Faktor",
            index=774,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ca10c191-383a-4394-9c37-c8daeb7abd4c"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="MIW",
            name="Messwandlersatz Strom",
            ahb_expression=None,
            section_name="Wandlertyp und Faktor",
            index=775,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="MIW",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c9f84f8e-b885-49dd-88a0-1bcde48751da"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="MPW",
            name="Kombimesswandlersatz (Strom und Spannung)",
            ahb_expression=None,
            section_name="Wandlertyp und Faktor",
            index=776,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="MPW",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("61dcc4bb-b57e-4814-9cfc-d698b600c7a7"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="MBW",
            name="Blockstromwandler",
            ahb_expression=None,
            section_name="Wandlertyp und Faktor",
            index=777,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="MBW",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f512f233-332e-4f55-a4f5-6f61e1250d38"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="MUW",
            name="Messwandlersatz Spannung",
            ahb_expression=None,
            section_name="Wandlertyp und Faktor",
            index=778,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="MUW",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b87f459c-2c91-45a3-b17a-b4a6748647d3"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry=None,
            name="Wandlerfaktor",
            ahb_expression=None,
            section_name="Wandlertyp und Faktor",
            index=779,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a65010e9-ac65-4fde-bdec-6b6529a9c3ff"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Kommunikationseinrichtung sdaten",
            index=781,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("92457075-5c7c-41a0-ac73-ebac57e54ae5"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Kommunikationseinrichtung sdaten",
            index=782,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("23b7d39e-ca2a-415b-8a93-0d01b8ce6ffc"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z05",
            name="Kommunikationseinrichtungsdaten",
            ahb_expression=None,
            section_name="Kommunikationseinrichtung sdaten",
            index=783,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z05",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3b86d54e-62b1-43ab-abcb-377daed894b1"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=785,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("009a6e74-75cf-4f42-9bca-53dd46b4c206"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=786,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("cf8643df-5772-43b8-9fcc-cae9c943663c"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="MG",
            name="Gerätenummer desZählers",
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=787,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="MG",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e86a4edc-561a-4a80-bcc9-8cf83370fc5f"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=788,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0d646926-dd2b-48f5-b332-37d985306d67"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Kommunikationseinrichtung",
            index=790,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("ad2bbb1c-5295-40af-abdd-a09f1febe921"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Kommunikationseinrichtung",
            index=791,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("fb8035b4-e003-4876-8434-1c8dd7e8d13f"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z26",
            name="Kommunikationseinrichtung",
            ahb_expression=None,
            section_name="Kommunikationseinrichtung",
            index=792,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z26",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("189da101-e103-4eef-9657-f5932bed796f"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=794,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2ba60247-adfd-4f36-bc79-2ce84b97a0f0"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=795,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d60431a3-64a2-4c80-8ed8-97a3fa7aef1a"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z30",
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=796,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z30",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("920cac92-db1b-420c-b8da-fea4a8443472"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry=None,
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=797,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8eb7787d-446c-4fa9-bb4d-ad9fb024089d"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Kommunikationseinrichtung styp",
            index=799,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1d350fb8-2d1c-4f3c-99ca-6af37fc84641"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Kommunikationseinrichtung styp",
            index=800,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("36e68cf6-dbd3-4c23-8762-f4073605dfae"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="GSM",
            name="GSM/GPRS/UMTS- Kom.-Einr.",
            ahb_expression=None,
            section_name="Kommunikationseinrichtung styp",
            index=801,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="GSM",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("189a7c6f-6a5b-4d3a-b817-4e2347920aba"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="ETH",
            name="Ethernet-Kom.-Einricht. LAN/WLAN",
            ahb_expression=None,
            section_name="Kommunikationseinrichtung styp",
            index=802,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="ETH",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("fc5719e5-5848-4ad0-b19e-6d455fdcc4a3"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="PLC",
            name="PLC-Kom.-Einrichtung",
            ahb_expression=None,
            section_name="Kommunikationseinrichtung styp",
            index=803,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="PLC",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1e7224ca-6c6a-4b7b-804f-ba9321587343"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="PST",
            name="Festnetz-Kom.-Einricht. TAE",
            ahb_expression=None,
            section_name="Kommunikationseinrichtung styp",
            index=804,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="PST",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("68de527f-2749-47a2-abd1-fe077052c1fc"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="DSL",
            name="DSL-Kom.Einr.",
            ahb_expression=None,
            section_name="Kommunikationseinrichtung styp",
            index=805,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="DSL",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e9010320-2ac4-45d3-930f-9c15a4c0b1ee"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="LTE",
            name="LTE-Kom.-Einr.",
            ahb_expression=None,
            section_name="Kommunikationseinrichtung styp",
            index=806,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="LTE",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("634f6e6a-eb82-4998-9ee0-d471a9fc2eb4"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Daten der technischen Steuereinrichtung",
            index=808,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("21ab87b5-f9f1-486c-a50a-0f32911c09bf"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Daten der technischen Steuereinrichtung",
            index=809,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4ce2e8ca-47fa-41c7-b5cf-7897b9be9a66"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z06",
            name="Daten der technischenSteuereinrichtung",
            ahb_expression=None,
            section_name="Daten der technischen Steuereinrichtung",
            index=810,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z06",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6675e9fe-0236-4f2e-a318-a93165d59015"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=812,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0a385c66-52f1-498e-9400-5d7b56afafcc"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=813,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("922d7c9f-77f0-48c4-a65e-f887b0a05e02"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="MG",
            name="Gerätenummer desZählers",
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=814,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="MG",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("95617052-708a-4f9e-baea-4c8fac094094"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Referenz auf die Gerätenummer",
            index=815,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8d34759f-57b9-46b8-8d21-a256d17e6968"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Technische Steuereinrichtung",
            index=817,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("feb7cb43-d80a-416f-85a0-fb021a7a0768"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Technische Steuereinrichtung",
            index=818,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("664bcb7e-d255-4ad7-bbe7-e1737400e65e"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z27",
            name="TechnischeSteuereinrichtung",
            ahb_expression=None,
            section_name="Technische Steuereinrichtung",
            index=819,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z27",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("242131d4-5167-4c96-8980-05d82266a8d4"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=821,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2fb073ad-63a5-46f5-a974-9a978afa195a"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=822,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("21513486-40fa-486a-bd1d-3bd142f337da"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z30",
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=823,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z30",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("362f92b5-deb4-40ab-a4dd-6fd2ffb901ef"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry=None,
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=824,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a4693ee9-5f40-4885-9432-5460daa23d95"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Technische Steuereinrichtungstyp",
            index=826,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("dcbf8f03-af52-471d-916d-a0156cb8238e"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Technische Steuereinrichtungstyp",
            index=827,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("fbb55d00-9885-43f4-ac2e-edeebdb3e5be"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="RSU",
            name="Rundsteuerempfänger",
            ahb_expression=None,
            section_name="Technische Steuereinrichtungstyp",
            index=828,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="RSU",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("25a5a016-4eb1-455c-860c-5c53fa1393ab"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="TSU",
            name="Tarifschaltuhr",
            ahb_expression=None,
            section_name="Technische Steuereinrichtungstyp",
            index=829,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="TSU",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("72804a5a-f70f-4a86-9fab-030012a53b82"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Smartmeter-Gateway",
            index=831,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("3b0c8973-4e01-40ac-99d4-1e8ca7f17fb8"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Smartmeter-Gateway",
            index=832,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("373d36ff-fb1b-4ad4-b891-4a191fa77720"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z13",
            name="Smartmeter-Gateway",
            ahb_expression=None,
            section_name="Smartmeter-Gateway",
            index=833,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z13",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a94c66a6-c81d-4757-a1da-2edb75712d85"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Smartmeter-Gateway",
            index=835,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z75"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("75e8f97b-8ef1-4c3b-bf7e-95bae7cbf191"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Smartmeter-Gateway",
            index=836,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z75"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5b435bc1-964b-4541-adfe-670ae88579e4"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z75",
            name="Smartmeter-Gateway",
            ahb_expression=None,
            section_name="Smartmeter-Gateway",
            index=837,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z75"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z75",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d42ee86e-32ce-4e72-bf7c-5312c9fdb601"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=839,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z75"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("16e1aff5-f934-4c40-ba98-093c4c058560"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=840,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z75"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9f0ee22b-a084-47b6-b292-24a34cf8f47a"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z30",
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=841,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z75"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z30",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2c43ea64-915b-41a1-a081-48c9203fd483"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry=None,
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=842,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z75"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("813b5afd-dbff-4a5d-9e32-f10028bc86fc"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Daten der Steuerbox",
            index=844,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("dd2c49e1-6352-4449-9da9-4fefebe5fd56"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Daten der Steuerbox",
            index=845,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ce42eb35-9414-4606-92fa-fb995807c019"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z14",
            name="Steuerbox",
            ahb_expression=None,
            section_name="Daten der Steuerbox",
            index=846,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z14",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8f3a17b7-dcf4-4549-8d80-ef413a224d3c"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf das Smartmeter-Gateway",
            index=848,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("360ec7c5-36c1-4578-b681-9cb95fe72b19"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf das Smartmeter-Gateway",
            index=849,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7a49a955-6c05-4441-af86-dc9e13158b07"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z14",
            name="Smartmeter-Gateway",
            ahb_expression=None,
            section_name="Referenz auf das Smartmeter-Gateway",
            index=850,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z14",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("cb782b87-ed60-456f-a94f-d01e7a1457d7"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="Identifikation / Nummer desSmartmeter-Gateways",
            ahb_expression=None,
            section_name="Referenz auf das Smartmeter-Gateway",
            index=851,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b1b497ed-b117-4144-91f3-10f96af5349e"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=853,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a53fd9f1-455e-49aa-92c2-87bbc1526aa2"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=854,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("412e9e5b-2767-49d9-96d7-821b7a9b8105"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z18",
            name="Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=855,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("43bc5f7f-ca4a-46b1-970f-2aa6ce8afabe"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID einer Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=856,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("82125dd4-1e86-467c-9338-a1968851843c"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Steuerbox",
            index=858,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z76"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("bbd0e2ae-bb4b-4374-831c-6ac1d9e25c9d"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Steuerbox",
            index=859,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z76"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f18585ab-5589-4979-95bb-5e522a91e190"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z76",
            name="Steuerbox",
            ahb_expression=None,
            section_name="Steuerbox",
            index=860,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z76"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z76",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("73bedad2-ab97-4684-8834-dc7eb2f1fda6"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=862,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z76"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a34d360d-0847-47af-b95a-ad5f7de91019"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=863,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z76"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("36480d7d-90ee-4989-8b05-cb3643c38f85"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry="Z30",
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=864,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z76"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier="Z30",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("48cfd07e-8060-42dd-abc2-fcd70a07ac96"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7110",
            value_pool_entry=None,
            name="Gerätenummer",
            ahb_expression=None,
            section_name="Identifikation / Nummer des Gerätes",
            index=865,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z76"),
            ],
            segment_code="CAV",
            data_element_id="7110",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("eb1de069-5e61-4b65-93b6-f971cdbd7c40"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Profildaten (Strom)",
            index=867,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("2c052688-9ddb-4107-8690-c79e9465941e"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Profildaten (Strom)",
            index=869,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6816e87f-e28c-42e3-932a-1d178a172ad7"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z21",
            name="Profildaten",
            ahb_expression=None,
            section_name="Profildaten (Strom)",
            index=870,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z21",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("18982b2c-414c-44e6-8430-9c8e77b18cc2"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=872,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0df5e37a-7e3a-45ac-849a-0125d4c369c7"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=873,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0bae6306-6736-461b-b6d0-996beace0974"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z18",
            name="Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=874,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1e3ef67f-2b66-4460-95a9-b1f2630c3622"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID einer Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=875,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c3277305-0ec4-4b1a-9f01-3fe4694441f5"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Normiertes Profil (Strom)",
            index=877,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("6d598acb-d3b9-4d05-9d0d-cf9bdde0917f"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Normiertes Profil (Strom)",
            index=878,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("0e3a83c7-43a8-4285-b16c-de1bfd0e8c7d"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z02",
            name="Standardlastprofil",
            ahb_expression=None,
            section_name="Normiertes Profil (Strom)",
            index=879,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z02",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e4535ec4-1a53-4b83-a72a-04f2219f7288"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z04",
            name="Standardeinspeiseprofil",
            ahb_expression=None,
            section_name="Normiertes Profil (Strom)",
            index=880,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z04",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6b7d9e90-db1b-4054-ab94-7c3f305e8546"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="E01",
            name="synthetisches Verfahren",
            ahb_expression=None,
            section_name="Normiertes Profil (Strom)",
            index=881,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="E01",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("dab48fb8-03cb-4b24-9ee4-f18c70aed82b"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z10",
            name="analytisches Verfahren",
            ahb_expression=None,
            section_name="Normiertes Profil (Strom)",
            index=882,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z10",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("23e3d39c-41fd-404e-adbd-b24fc3c2b863"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Normiertes Profil (Strom)",
            index=884,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b7fdcde0-e485-457c-9f7d-380edf019a9c"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Normiertes Profil (Strom)",
            index=885,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5d3f8098-8aa1-4ebd-92a2-a067d49bcfa4"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry=None,
            name="Code des Normierten Profils",
            ahb_expression=None,
            section_name="Normiertes Profil (Strom)",
            index=886,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("acfdce3d-8efd-4674-a273-76a1b462864b"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Profilschardaten",
            index=888,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("36a32cc6-67c5-4a5f-bf3c-c182035fd1d6"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Profilschardaten",
            index=889,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4fd0298a-6f65-4658-8514-85a88be3cc03"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z08",
            name="Profilschardaten",
            ahb_expression=None,
            section_name="Profilschardaten",
            index=890,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z08",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9f1727e4-1896-43c7-bb14-3db8523ca818"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=892,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c6cdb778-b455-402e-9f25-77c01a9ddcf4"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=893,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("47bde71a-2843-4b97-8e16-747b700d0725"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z18",
            name="Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=894,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("eb1ad19d-50f1-454d-9f7c-3d7e05acb886"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID einer Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=895,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e3c0396e-8340-4fb3-a0ce-c8536ece85f3"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Normiertes Profil",
            index=897,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("9a178c8b-053a-46bc-b68d-2a9442a4389b"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Normiertes Profil",
            index=898,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9c617dfc-e920-4874-8bba-313068f9128e"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z03",
            name="tagesparameterabhängig es Lastprofil",
            ahb_expression=None,
            section_name="Normiertes Profil",
            index=899,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z03",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4a8e0780-238a-4203-9c46-e603fc1cb78f"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z05",
            name="tagesparameterabhängig es Einspeiseprofil",
            ahb_expression=None,
            section_name="Normiertes Profil",
            index=900,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z05",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("01f17cb4-8a2f-4442-8cf6-5076cf92595a"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="E01",
            name="synthetisches Verfahren",
            ahb_expression=None,
            section_name="Normiertes Profil",
            index=901,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="E01",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("96d8ce78-19c4-4b40-96ba-d0ba0a439b0f"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry="Z10",
            name="analytisches Verfahren",
            ahb_expression=None,
            section_name="Normiertes Profil",
            index=902,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier="Z10",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("69a5f0b4-348b-4844-a25b-300bddd1fe41"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Normiertes Profil",
            index=904,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a914dbb4-9369-4b9b-a2f9-463b159661d1"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Normiertes Profil",
            index=905,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6d04b477-aabd-4865-9e22-44cfad340351"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry=None,
            name="Code des Normierten Profils(Strom)",
            ahb_expression=None,
            section_name="Normiertes Profil",
            index=906,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2e36aba6-850b-40d4-948c-040d88421aae"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Klimazone / Temperaturmessstelle",
            index=908,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("c93906d7-c16f-4035-a9e0-9b16fb92cb37"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Klimazone / Temperaturmessstelle",
            index=909,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("05bb8b59-deb0-49ec-8f92-c506be57b26e"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z99",
            name="Messstelle des Tagesparameters (derzeit ist nur die Temperatur ein erlaubter Tagesparameter)",
            ahb_expression=None,
            section_name="Klimazone / Temperaturmessstelle",
            index=910,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z99",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4177afba-ec78-4aba-be33-d2320b357f53"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="ZA0",
            name="Klimazone des Tagesparameters (derzeit ist Tagesparameter)",
            ahb_expression=None,
            section_name="Klimazone / Temperaturmessstelle",
            index=911,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="ZA0",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("16acdc61-99b3-4588-abaf-08b9dd8ab5f8"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7037",
            value_pool_entry=None,
            name="ID Klimazone /Temperaturmessstelle",
            ahb_expression=None,
            section_name="Klimazone / Temperaturmessstelle",
            index=912,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
            ],
            segment_code="CCI",
            data_element_id="7037",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e4132627-b679-4725-a1a6-d2eed69717df"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="1131",
            value_pool_entry=None,
            name="Dienstanbieter",
            ahb_expression=None,
            section_name="Klimazone / Temperaturmessstelle",
            index=913,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
            ],
            segment_code="CCI",
            data_element_id="1131",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("21f8fc39-f218-490c-99e3-83b746490c5a"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="3055",
            value_pool_entry="89",
            name="Vergeben vom Händler (hier Netzbetreiber)",
            ahb_expression=None,
            section_name="Klimazone / Temperaturmessstelle",
            index=914,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
            ],
            segment_code="CCI",
            data_element_id="3055",
            qualifier="89",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3be124bb-bc43-448f-89d4-918ab47384e7"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="3055",
            value_pool_entry="293",
            name="DE, BDEW (Bundesverband der Energie- und Wasserwirtschaft e.V.)",
            ahb_expression=None,
            section_name="Klimazone / Temperaturmessstelle",
            index=915,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
            ],
            segment_code="CCI",
            data_element_id="3055",
            qualifier="293",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a8e819cb-6c15-4900-ab48-092ae3786433"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenzprofildaten",
            index=917,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("f30158f3-d888-41c2-96ec-114203146d13"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenzprofildaten",
            index=919,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
            ],
            segment_code="SEQ",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("98e3aae1-b27a-4b7e-b879-8143c8ba3bc4"),
            segment_group_key="SG8",
            segment_code="SEQ",
            data_element="1229",
            value_pool_entry="Z38",
            name="Referenzprofildaten",
            ahb_expression=None,
            section_name="Referenzprofildaten",
            index=920,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
            ],
            segment_code="SEQ",
            data_element_id="1229",
            qualifier="Z38",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("13547646-a2f2-46be-963d-f77f3ed90707"),
            segment_group_key="SG8",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=922,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b11656ad-143a-4c81-9d93-07023c69dae2"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=923,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f3aeab5b-2354-4908-b6c3-45e42e4750a6"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z18",
            name="Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=924,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c7a9d405-157d-4da1-8a60-f9ff4342f45d"),
            segment_group_key="SG8",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID einer Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=925,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d0bf5863-92d2-41ae-a9fa-528fdc8e2487"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenzprofil",
            index=927,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z05"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("a8f00471-0907-41de-8de7-7e4dd1c97c55"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenzprofil",
            index=928,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z05"),
            ],
            segment_code="CCI",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e994cf84-b92a-49dc-9833-b9dd037abc02"),
            segment_group_key="SG10",
            segment_code="CCI",
            data_element="7059",
            value_pool_entry="Z05",
            name="tagesparameterabhängiges Einspeiseprofil",
            ahb_expression=None,
            section_name="Referenzprofil",
            index=929,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z05"),
            ],
            segment_code="CCI",
            data_element_id="7059",
            qualifier="Z05",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d2868bf2-3561-4425-a33f-07c9ccc828db"),
            segment_group_key="SG10",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenzprofil",
            index=931,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z05"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("233e321f-608c-4874-85a1-0712a96fda4f"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenzprofil",
            index=932,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z05"),
            ],
            segment_code="CAV",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7dffce65-397d-4a82-b0ae-50978f8dbda5"),
            segment_group_key="SG10",
            segment_code="CAV",
            data_element="7111",
            value_pool_entry=None,
            name="Code des Referenzprofils",
            ahb_expression=None,
            section_name="Referenzprofil",
            index=933,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
                AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z05"),
            ],
            segment_code="CAV",
            data_element_id="7111",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("404a0284-68f5-4100-b443-884ec033e024"),
            segment_group_key="SG12",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Kunde des Lieferanten",
            index=935,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("3af26b26-4436-4fca-8b84-3897cc9d33e2"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Kunde des Lieferanten",
            index=936,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("d3cbe1d0-a207-4515-ac25-2dfd35609390"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3035",
            value_pool_entry="Z09",
            name="Kunde des LF",
            ahb_expression=None,
            section_name="Kunde des Lieferanten",
            index=937,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
            ],
            segment_code="NAD",
            data_element_id="3035",
            qualifier="Z09",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ad3e8edf-f059-41ac-93b7-3c9014f9aabb"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3036",
            value_pool_entry=None,
            name="Name",
            ahb_expression=None,
            section_name="Kunde des Lieferanten",
            index=938,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
            ],
            segment_code="NAD",
            data_element_id="3036",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e16acf19-0817-4f3c-8bad-3d7f219c30db"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3045",
            value_pool_entry="Z01",
            name="Struktur von Personennamen",
            ahb_expression=None,
            section_name="Kunde des Lieferanten",
            index=939,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
            ],
            segment_code="NAD",
            data_element_id="3045",
            qualifier="Z01",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("31298092-3477-4234-b354-0c5f2cdefc7c"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3045",
            value_pool_entry="Z02",
            name="Struktur der Firmenbezeichnung",
            ahb_expression=None,
            section_name="Kunde des Lieferanten",
            index=940,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
            ],
            segment_code="NAD",
            data_element_id="3045",
            qualifier="Z02",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("bb9b2021-0ec7-4a6e-a9e0-ec66e2d431e7"),
            segment_group_key="SG12",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=942,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("597457cf-7be8-4509-b846-7b9444f776ea"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=943,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f59da53b-4079-4bb3-9897-616fd50d3907"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z18",
            name="Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=944,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("efbb74f2-1816-42b9-9a08-7b5858cc1f2f"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID der Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=945,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("89224b5d-af0a-47f8-9733-733c60ef5a0e"),
            segment_group_key="SG12",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Korrespondenzanschrift des Kunden des Lieferanten",
            index=947,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("cbd3b6bb-02e4-48ed-be08-0c6803e3328b"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Korrespondenzanschrift des Kunden des Lieferanten",
            index=949,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("040e1b2c-1b7c-434a-bf07-12db1a41e33f"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3035",
            value_pool_entry="Z04",
            name="Korrespondenzanschriftdes Kunden des LF",
            ahb_expression=None,
            section_name="Korrespondenzanschrift des Kunden des Lieferanten",
            index=950,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
            ],
            segment_code="NAD",
            data_element_id="3035",
            qualifier="Z04",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("49509560-3005-4365-bd1f-ef2b7c7e0962"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3036",
            value_pool_entry=None,
            name="Name",
            ahb_expression=None,
            section_name="Korrespondenzanschrift des Kunden des Lieferanten",
            index=951,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
            ],
            segment_code="NAD",
            data_element_id="3036",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("7849312c-4921-4d2b-9111-ae331f0a7f85"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3045",
            value_pool_entry="Z01",
            name="Struktur von Personennamen",
            ahb_expression=None,
            section_name="Korrespondenzanschrift des Kunden des Lieferanten",
            index=952,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
            ],
            segment_code="NAD",
            data_element_id="3045",
            qualifier="Z01",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9b248a47-8c45-4f02-9195-cefe895011e2"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3045",
            value_pool_entry="Z02",
            name="Struktur der Firmenbezeichnung",
            ahb_expression=None,
            section_name="Korrespondenzanschrift des Kunden des Lieferanten",
            index=953,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
            ],
            segment_code="NAD",
            data_element_id="3045",
            qualifier="Z02",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e46e732b-0e65-4391-b2a8-b3480e896fc9"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3042",
            value_pool_entry=None,
            name="Straße und Hausnummer oderPostfach",
            ahb_expression=None,
            section_name="Korrespondenzanschrift des Kunden des Lieferanten",
            index=954,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
            ],
            segment_code="NAD",
            data_element_id="3042",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("00bcc545-8530-4892-9b67-683cee34faf9"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3164",
            value_pool_entry=None,
            name="Ort",
            ahb_expression=None,
            section_name="Korrespondenzanschrift des Kunden des Lieferanten",
            index=955,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
            ],
            segment_code="NAD",
            data_element_id="3164",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1d9bf524-73ce-4a5d-ae33-f1efd2a1dbbc"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3251",
            value_pool_entry=None,
            name="Postleitzahl",
            ahb_expression=None,
            section_name="Korrespondenzanschrift des Kunden des Lieferanten",
            index=956,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
            ],
            segment_code="NAD",
            data_element_id="3251",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("4da1416f-f3cc-4384-b637-a44f54d72633"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3207",
            value_pool_entry=None,
            name="Ländername, Code",
            ahb_expression=None,
            section_name="Korrespondenzanschrift des Kunden des Lieferanten",
            index=957,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
            ],
            segment_code="NAD",
            data_element_id="3207",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6389e36d-0613-4fed-ac28-be2f866c634b"),
            segment_group_key="SG12",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=959,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6a096f09-f95b-4c10-8d14-235337cc1cd9"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=960,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("478df47b-14af-4c0a-9661-9bf3f9e632f3"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z18",
            name="Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=961,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5e8ab032-09cc-4bc5-abd1-6ae0561bac22"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID der Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=962,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("287a5fdd-d6c8-48bb-830d-947e1bffba68"),
            segment_group_key="SG12",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Kunde des Messstellenbetreibers",
            index=964,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("e4a788e9-1f57-4721-b9f7-8c3049060900"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Kunde des Messstellenbetreibers",
            index=965,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("587bbc8e-1ff3-43d7-a683-5c5af8167363"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3035",
            value_pool_entry="Z07",
            name="Kunde des MSB",
            ahb_expression="X",
            section_name="Kunde des Messstellenbetreibers",
            index=966,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
            ],
            segment_code="NAD",
            data_element_id="3035",
            qualifier="Z07",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a3b1d536-9cf2-4cbd-9796-fc8d62d1e328"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3036",
            value_pool_entry=None,
            name="Name",
            ahb_expression="X",
            section_name="Kunde des Messstellenbetreibers",
            index=967,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
            ],
            segment_code="NAD",
            data_element_id="3036",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("001b239c-c930-4878-ad22-a48b5122ae7d"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3045",
            value_pool_entry="Z01",
            name="Struktur von Personennamen",
            ahb_expression="X",
            section_name="Kunde des Messstellenbetreibers",
            index=968,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
            ],
            segment_code="NAD",
            data_element_id="3045",
            qualifier="Z01",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5896dd4b-bdee-4bba-9f7e-ab30101745a5"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3045",
            value_pool_entry="Z02",
            name="Struktur der Firmenbezeichnung",
            ahb_expression="X",
            section_name="Kunde des Messstellenbetreibers",
            index=969,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
            ],
            segment_code="NAD",
            data_element_id="3045",
            qualifier="Z02",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f2c00ee6-5a56-45ea-9721-b298c92905ed"),
            segment_group_key="SG12",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Messlokation",
            index=971,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9c477992-0b91-49fc-af6c-0dc055b1821a"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Messlokation",
            index=972,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b2b81b83-dde8-436c-90e4-d673bb52a091"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z19",
            name="Messlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Messlokation",
            index=973,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z19",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1450190b-2c15-4672-9847-e9148b38ec78"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID der Messlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Messlokation",
            index=974,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("eb4044b8-ddef-4b02-81b8-523208ef5398"),
            segment_group_key="SG12",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Korrespondenzanschrift des Kunden des Messstellenbetreibers",
            index=976,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("b8910f39-31cd-42d1-89c7-2d24c41af2b5"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Korrespondenzanschrift des Kunden des Messstellenbetreibers",
            index=977,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("df885620-8230-4845-9607-1a22b773580f"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3035",
            value_pool_entry="Z08",
            name="Korrespondenzanschriftdes Kunden des MSB",
            ahb_expression="X",
            section_name="Korrespondenzanschrift des Kunden des Messstellenbetreibers",
            index=978,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
            ],
            segment_code="NAD",
            data_element_id="3035",
            qualifier="Z08",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5a931234-5365-4eec-a3ba-1251af448ceb"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3036",
            value_pool_entry=None,
            name="Name",
            ahb_expression="X",
            section_name="Korrespondenzanschrift des Kunden des Messstellenbetreibers",
            index=979,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
            ],
            segment_code="NAD",
            data_element_id="3036",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a98200f4-1153-4ead-8f1d-10d2d1cbe1fd"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3045",
            value_pool_entry="Z01",
            name="Struktur von Personennamen",
            ahb_expression="X",
            section_name="Korrespondenzanschrift des Kunden des Messstellenbetreibers",
            index=980,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
            ],
            segment_code="NAD",
            data_element_id="3045",
            qualifier="Z01",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f75e4ea9-472a-41df-b933-4fefe181638c"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3045",
            value_pool_entry="Z02",
            name="Struktur der Firmenbezeichnung",
            ahb_expression="X",
            section_name="Korrespondenzanschrift des Kunden des Messstellenbetreibers",
            index=981,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
            ],
            segment_code="NAD",
            data_element_id="3045",
            qualifier="Z02",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("fe3a6651-51e1-402b-b4a9-a8a3199dd0d7"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3042",
            value_pool_entry=None,
            name="Straße und Hausnummer oderPostfach",
            ahb_expression="X",
            section_name="Korrespondenzanschrift des Kunden des Messstellenbetreibers",
            index=982,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
            ],
            segment_code="NAD",
            data_element_id="3042",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2e4db692-4478-480d-bab3-746d958dbdb0"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3164",
            value_pool_entry=None,
            name="Ort",
            ahb_expression="X",
            section_name="Korrespondenzanschrift des Kunden des Messstellenbetreibers",
            index=983,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
            ],
            segment_code="NAD",
            data_element_id="3164",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("dd577c7f-32c8-4c47-a7e3-9a3feacd80fb"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3251",
            value_pool_entry=None,
            name="Postleitzahl",
            ahb_expression="M [268]S [166]",
            section_name="Korrespondenzanschrift des Kunden des Messstellenbetreibers",
            index=984,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
            ],
            segment_code="NAD",
            data_element_id="3251",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8a4b3213-5e51-448e-a729-0759c1948471"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3207",
            value_pool_entry=None,
            name="Ländername, Code",
            ahb_expression="X",
            section_name="Korrespondenzanschrift des Kunden des Messstellenbetreibers",
            index=985,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
            ],
            segment_code="NAD",
            data_element_id="3207",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3036e94f-085a-4124-8ed3-df558db3bbab"),
            segment_group_key="SG12",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=987,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("ee0e9f62-7fb2-4eda-88b5-f69420ab8171"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=988,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3c3379c8-d34c-4e21-9e4c-ce79609a8fd6"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z18",
            name="Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=989,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z18",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3c1e0439-a4d8-4a90-82c2-e3063e2b6392"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID der Marktlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID der Marktlokation",
            index=990,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3252c715-7c46-44b6-889d-6c9f97fbf3c2"),
            segment_group_key="SG12",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Soll [165]",
            section_name="Messlokationsadresse",
            index=992,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("26b4ea71-58cd-4703-a272-5167622e26b8"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Messlokationsadresse",
            index=993,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("65bb27a2-2b13-40eb-84e0-43c29a8dee66"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3035",
            value_pool_entry="Z03",
            name="Messlokationsadresse",
            ahb_expression="X",
            section_name="Messlokationsadresse",
            index=994,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
            ],
            segment_code="NAD",
            data_element_id="3035",
            qualifier="Z03",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("82efcc4c-c0eb-41aa-80fb-35419c7c9ec0"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3124",
            value_pool_entry=None,
            name="Zusatzinformation zurIdentifizierung",
            ahb_expression="K",
            section_name="Messlokationsadresse",
            index=995,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
            ],
            segment_code="NAD",
            data_element_id="3124",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("709cc9fb-eebb-4b75-a3df-4cd41aeb6d0c"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3042",
            value_pool_entry=None,
            name="Straße und Hausnummer oderPostfach",
            ahb_expression="S [166]M [212]",
            section_name="Messlokationsadresse",
            index=996,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
            ],
            segment_code="NAD",
            data_element_id="3042",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("60e2e879-dcd6-4a74-a4b6-32eb0195552d"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3164",
            value_pool_entry=None,
            name="Ort",
            ahb_expression="X",
            section_name="Messlokationsadresse",
            index=997,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
            ],
            segment_code="NAD",
            data_element_id="3164",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("90e64654-d69c-4648-bb3e-c3c54cc1c702"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3251",
            value_pool_entry=None,
            name="Postleitzahl",
            ahb_expression="X",
            section_name="Messlokationsadresse",
            index=998,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
            ],
            segment_code="NAD",
            data_element_id="3251",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("8e94fafa-b3b0-4f7b-b55b-ce91094a2a29"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3207",
            value_pool_entry=None,
            name="Ländername, Code",
            ahb_expression="X",
            section_name="Messlokationsadresse",
            index=999,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
            ],
            segment_code="NAD",
            data_element_id="3207",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("9ce5325f-b8b7-41d1-9518-c775f59713c7"),
            segment_group_key="SG12",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID einer Messlokation",
            index=1001,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("26153d40-d3fc-43d2-822e-700a3488c338"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID einer Messlokation",
            index=1002,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5c97843b-251a-4003-b341-13e55e91a604"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z19",
            name="Messlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID einer Messlokation",
            index=1003,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z19",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("01c029e8-e0ec-4c22-9966-10e12ea73575"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID einer Messlokation",
            ahb_expression=None,
            section_name="Referenz auf die ID einer Messlokation",
            index=1004,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("559f5f4a-0231-4725-b50f-86c41bfeaa96"),
            segment_group_key="SG12",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Name und Adresse für die Ablesekarte",
            index=1006,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.MOVE_TO_NEIGHBOUR_SAME_KEY,
    ),
    (
        AhbLine(
            guid=UUID("0a608d62-91e3-4826-b312-10474f0744ae"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Name und Adresse für die Ablesekarte",
            index=1007,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
            ],
            segment_code="NAD",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("e6e50254-5496-4bce-8603-72c2cd92c1c8"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3035",
            value_pool_entry="Z05",
            name="Name und Adresse fürdie Ablesekarte",
            ahb_expression="X",
            section_name="Name und Adresse für die Ablesekarte",
            index=1008,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
            ],
            segment_code="NAD",
            data_element_id="3035",
            qualifier="Z05",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c3c0d71b-b221-4e21-98a3-6686f9dd9eaf"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3036",
            value_pool_entry=None,
            name="Name",
            ahb_expression="X",
            section_name="Name und Adresse für die Ablesekarte",
            index=1009,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
            ],
            segment_code="NAD",
            data_element_id="3036",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("c04eb265-447b-46f7-bb2a-aa9923808d32"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3045",
            value_pool_entry="Z01",
            name="Struktur von Personennamen",
            ahb_expression="X",
            section_name="Name und Adresse für die Ablesekarte",
            index=1010,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
            ],
            segment_code="NAD",
            data_element_id="3045",
            qualifier="Z01",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f15730aa-a895-4e87-8e0a-e1fec0eeac4c"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3045",
            value_pool_entry="Z02",
            name="Struktur der Firmenbezeichnung",
            ahb_expression="X",
            section_name="Name und Adresse für die Ablesekarte",
            index=1011,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
            ],
            segment_code="NAD",
            data_element_id="3045",
            qualifier="Z02",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("3829e064-9fa0-4d1f-b9c7-77d1119ec11a"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3042",
            value_pool_entry=None,
            name="Straße und Hausnummer oderPostfach",
            ahb_expression="X",
            section_name="Name und Adresse für die Ablesekarte",
            index=1012,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
            ],
            segment_code="NAD",
            data_element_id="3042",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("dec666b9-55ea-48f4-b0df-6b41712e9af1"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3164",
            value_pool_entry=None,
            name="Ort",
            ahb_expression="X",
            section_name="Name und Adresse für die Ablesekarte",
            index=1013,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
            ],
            segment_code="NAD",
            data_element_id="3164",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("640ab3b7-4d59-437b-aaf4-889ac30defa9"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3251",
            value_pool_entry=None,
            name="Postleitzahl",
            ahb_expression="M [268]S [166]",
            section_name="Name und Adresse für die Ablesekarte",
            index=1014,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
            ],
            segment_code="NAD",
            data_element_id="3251",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("f56180e1-375a-4fa4-92ea-2a7c5079756e"),
            segment_group_key="SG12",
            segment_code="NAD",
            data_element="3207",
            value_pool_entry=None,
            name="Ländername, Code",
            ahb_expression="X",
            section_name="Name und Adresse für die Ablesekarte",
            index=1015,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
            ],
            segment_code="NAD",
            data_element_id="3207",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("b8de2901-ab1a-458e-8741-039427d0f411"),
            segment_group_key="SG12",
            segment_code=None,
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression=None,
            section_name="Referenz auf die ID einer Messlokation",
            index=1017,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("2fa31394-80af-4567-a07c-4c7791454373"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Kann",
            section_name="Referenz auf die ID einer Messlokation",
            index=1018,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
            ],
            segment_code="RFF",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("6527aff7-6aa6-48b3-a4d6-f3be58fb2dc4"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element="1153",
            value_pool_entry="Z19",
            name="Messlokation",
            ahb_expression="X",
            section_name="Referenz auf die ID einer Messlokation",
            index=1019,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
            ],
            segment_code="RFF",
            data_element_id="1153",
            qualifier="Z19",
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("1a709e96-2105-4d1c-a2cf-b1a57f57a0e2"),
            segment_group_key="SG12",
            segment_code="RFF",
            data_element="1154",
            value_pool_entry=None,
            name="ID einer Messlokation",
            ahb_expression="X [951]",
            section_name="Referenz auf die ID einer Messlokation",
            index=1020,
        ),
        AhbLocation(
            layers=[
                AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
                AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
                AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
            ],
            segment_code="RFF",
            data_element_id="1154",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("a0297258-bedf-4a36-a32a-0d93e84354ba"),
            segment_group_key=None,
            segment_code="UNT",
            data_element=None,
            value_pool_entry=None,
            name=None,
            ahb_expression="Muss",
            section_name="Nachrichten-Endesegment",
            index=1022,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNT", opening_qualifier=None)],
            segment_code="UNT",
            data_element_id=None,
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.LEAVE_TO_PARENT_AND_DIVE_INTO_SUB_GROUP,
    ),
    (
        AhbLine(
            guid=UUID("82624fe3-2fb2-4b48-815e-0c1784aa5d05"),
            segment_group_key=None,
            segment_code="UNT",
            data_element="0074",
            value_pool_entry=None,
            name="Anzahl der Segmente in einerNachricht",
            ahb_expression="X",
            section_name="Nachrichten-Endesegment",
            index=1023,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNT", opening_qualifier=None)],
            segment_code="UNT",
            data_element_id="0074",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
    (
        AhbLine(
            guid=UUID("5b822b8c-7dc9-4e41-a5ab-f3d85b076e76"),
            segment_group_key=None,
            segment_code="UNT",
            data_element="0062",
            value_pool_entry=None,
            name="Nachrichten-Referenznummer",
            ahb_expression="X",
            section_name="Nachrichten-Endesegment",
            index=1024,
        ),
        AhbLocation(
            layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNT", opening_qualifier=None)],
            segment_code="UNT",
            data_element_id="0062",
            qualifier=None,
        ),
        _DifferentialAhbLineHierarchyChange.STAY,
    ),
]

expected_locations_11042 = [x[1] for x in lines_locations_changes_11042]
expected_changes_11042 = [x[2] for x in lines_locations_changes_11042]

_ = [
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
        segment_code="UNH",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
        segment_code="UNH",
        data_element_id="0062",
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
        segment_code="UNH",
        data_element_id="0065",
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
        segment_code="UNH",
        data_element_id="0052",
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
        segment_code="UNH",
        data_element_id="0054",
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
        segment_code="UNH",
        data_element_id="0051",
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
        segment_code="UNH",
        data_element_id="0057",
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
        segment_code="BGM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
        segment_code="BGM",
        data_element_id="1001",
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
        segment_code="BGM",
        data_element_id="1004",
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
        segment_code="DTM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
        segment_code="DTM",
        data_element_id="2005",
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
        segment_code="DTM",
        data_element_id="2380",
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD")],
        segment_code="DTM",
        data_element_id="2379",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
        ],
        segment_code="NAD",
        data_element_id="3035",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
        ],
        segment_code="NAD",
        data_element_id="3039",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
        ],
        segment_code="NAD",
        data_element_id="3055",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
        ],
        segment_code="NAD",
        data_element_id="3055",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
        ],
        segment_code="NAD",
        data_element_id="3055",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
        ],
        segment_code="CTA",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
        ],
        segment_code="CTA",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
        ],
        segment_code="CTA",
        data_element_id="3139",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
        ],
        segment_code="CTA",
        data_element_id="3412",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
        ],
        segment_code="COM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
        ],
        segment_code="COM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
        ],
        segment_code="COM",
        data_element_id="3148",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
        ],
        segment_code="COM",
        data_element_id="3155",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
        ],
        segment_code="COM",
        data_element_id="3155",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
        ],
        segment_code="COM",
        data_element_id="3155",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
        ],
        segment_code="COM",
        data_element_id="3155",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MS"),
            AhbLocationLayer(segment_group_key="SG3", opening_segment_code="CTA", opening_qualifier="IC"),
        ],
        segment_code="COM",
        data_element_id="3155",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MR"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MR"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MR"),
        ],
        segment_code="NAD",
        data_element_id="3035",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MR"),
        ],
        segment_code="NAD",
        data_element_id="3039",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MR"),
        ],
        segment_code="NAD",
        data_element_id="3055",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MR"),
        ],
        segment_code="NAD",
        data_element_id="3055",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG2", opening_segment_code="NAD", opening_qualifier="MR"),
        ],
        segment_code="NAD",
        data_element_id="3055",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="IDE",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="IDE",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="IDE",
        data_element_id="7495",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="IDE",
        data_element_id="7402",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="DTM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="DTM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="DTM",
        data_element_id="2005",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="DTM",
        data_element_id="2380",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="DTM",
        data_element_id="2379",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id="9015",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id="9013",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id="9013",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id="9013",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id="9013",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id="9015",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id="9013",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id="1131",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id="1131",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id="1131",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="STS",
        data_element_id="1131",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="AGR",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="AGR",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="AGR",
        data_element_id="7431",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="AGR",
        data_element_id="7433",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
        ],
        segment_code="AGR",
        data_element_id="7433",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG5", opening_segment_code="LOC", opening_qualifier="172"),
        ],
        segment_code="LOC",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG5", opening_segment_code="LOC", opening_qualifier="172"),
        ],
        segment_code="LOC",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG5", opening_segment_code="LOC", opening_qualifier="172"),
        ],
        segment_code="LOC",
        data_element_id="3227",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG5", opening_segment_code="LOC", opening_qualifier="172"),
        ],
        segment_code="LOC",
        data_element_id="3225",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z13"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z13"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z13"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z13"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z13"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z13"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="TN"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="TN"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="TN"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="TN"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id="2005",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id="2380",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id="2379",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id="2379",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id="2005",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id="2380",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id="2379",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id="2005",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id="2380",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z18"),
        ],
        segment_code="DTM",
        data_element_id="2379",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="DTM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="DTM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="DTM",
        data_element_id="2005",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="DTM",
        data_element_id="2380",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="DTM",
        data_element_id="2379",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="DTM",
        data_element_id="2379",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="DTM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="DTM",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="DTM",
        data_element_id="2005",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="DTM",
        data_element_id="2380",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG6", opening_segment_code="RFF", opening_qualifier="Z19"),
        ],
        segment_code="DTM",
        data_element_id="2379",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="265"),
        ],
        segment_code="QTY",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="265"),
        ],
        segment_code="QTY",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="265"),
        ],
        segment_code="QTY",
        data_element_id="6063",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="265"),
        ],
        segment_code="QTY",
        data_element_id="6063",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="265"),
        ],
        segment_code="QTY",
        data_element_id="6060",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="265"),
        ],
        segment_code="QTY",
        data_element_id="6411",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="265"),
        ],
        segment_code="QTY",
        data_element_id="6411",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="31"),
        ],
        segment_code="QTY",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="31"),
        ],
        segment_code="QTY",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="31"),
        ],
        segment_code="QTY",
        data_element_id="6063",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="31"),
        ],
        segment_code="QTY",
        data_element_id="6060",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="31"),
        ],
        segment_code="QTY",
        data_element_id="6411",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z30"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z30"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z30"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z30"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z30"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="1131",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="1131",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="1131",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E03"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z83"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z83"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z83"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z83"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z83"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z83"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z83"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="6"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="6"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="6"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="6"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="6"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZC0"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z34"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z36"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z36"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z01"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z36"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z27"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
        ],
        segment_code="PIA",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
        ],
        segment_code="PIA",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
        ],
        segment_code="PIA",
        data_element_id="4347",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
        ],
        segment_code="PIA",
        data_element_id="7140",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
        ],
        segment_code="PIA",
        data_element_id="7143",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z38"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z38"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z38"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z38"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z02"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="11"),
        ],
        segment_code="QTY",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="11"),
        ],
        segment_code="QTY",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="11"),
        ],
        segment_code="QTY",
        data_element_id="6063",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="11"),
        ],
        segment_code="QTY",
        data_element_id="6060",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG9", opening_segment_code="QTY", opening_qualifier="11"),
        ],
        segment_code="QTY",
        data_element_id="6411",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="1131",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z37"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z37"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z37"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z37"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z15"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z37"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA8"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA7"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z16"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZA9"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="1131",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="ZB3"),
        ],
        segment_code="CAV",
        data_element_id="1131",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E04"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Y02"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Y02"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Y02"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Y02"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Y02"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Y02"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Y02"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z01"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z01"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z01"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z01"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z01"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z32"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z32"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z32"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z32"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z32"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z18"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z32"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z35"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z19"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z39"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E13"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z28"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E12"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E12"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E12"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E12"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E12"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E12"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z03"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="E12"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z09"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z64"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="PIA",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="PIA",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="PIA",
        data_element_id="4347",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="PIA",
        data_element_id="7140",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
        ],
        segment_code="PIA",
        data_element_id="7143",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z20"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="11"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z04"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z25"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z05"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z26"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z06"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z27"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z75"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z75"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z75"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z75"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z75"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z75"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z13"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z75"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z76"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z76"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z76"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z76"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z76"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z76"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z14"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z76"),
        ],
        segment_code="CAV",
        data_element_id="7110",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z21"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z02"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z03"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
        ],
        segment_code="CCI",
        data_element_id="7037",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
        ],
        segment_code="CCI",
        data_element_id="1131",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
        ],
        segment_code="CCI",
        data_element_id="3055",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z08"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z99"),
        ],
        segment_code="CCI",
        data_element_id="3055",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
        ],
        segment_code="SEQ",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
        ],
        segment_code="SEQ",
        data_element_id="1229",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z05"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z05"),
        ],
        segment_code="CCI",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z05"),
        ],
        segment_code="CCI",
        data_element_id="7059",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z05"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z05"),
        ],
        segment_code="CAV",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG8", opening_segment_code="SEQ", opening_qualifier="Z38"),
            AhbLocationLayer(segment_group_key="SG10", opening_segment_code="CCI", opening_qualifier="Z05"),
        ],
        segment_code="CAV",
        data_element_id="7111",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
        ],
        segment_code="NAD",
        data_element_id="3035",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
        ],
        segment_code="NAD",
        data_element_id="3036",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
        ],
        segment_code="NAD",
        data_element_id="3045",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
        ],
        segment_code="NAD",
        data_element_id="3045",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z09"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
        ],
        segment_code="NAD",
        data_element_id="3035",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
        ],
        segment_code="NAD",
        data_element_id="3036",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
        ],
        segment_code="NAD",
        data_element_id="3045",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
        ],
        segment_code="NAD",
        data_element_id="3045",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
        ],
        segment_code="NAD",
        data_element_id="3042",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
        ],
        segment_code="NAD",
        data_element_id="3164",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
        ],
        segment_code="NAD",
        data_element_id="3251",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
        ],
        segment_code="NAD",
        data_element_id="3207",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z04"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
        ],
        segment_code="NAD",
        data_element_id="3035",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
        ],
        segment_code="NAD",
        data_element_id="3036",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
        ],
        segment_code="NAD",
        data_element_id="3045",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
        ],
        segment_code="NAD",
        data_element_id="3045",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z07"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
        ],
        segment_code="NAD",
        data_element_id="3035",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
        ],
        segment_code="NAD",
        data_element_id="3036",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
        ],
        segment_code="NAD",
        data_element_id="3045",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
        ],
        segment_code="NAD",
        data_element_id="3045",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
        ],
        segment_code="NAD",
        data_element_id="3042",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
        ],
        segment_code="NAD",
        data_element_id="3164",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
        ],
        segment_code="NAD",
        data_element_id="3251",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
        ],
        segment_code="NAD",
        data_element_id="3207",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z08"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
        ],
        segment_code="NAD",
        data_element_id="3035",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
        ],
        segment_code="NAD",
        data_element_id="3124",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
        ],
        segment_code="NAD",
        data_element_id="3042",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
        ],
        segment_code="NAD",
        data_element_id="3164",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
        ],
        segment_code="NAD",
        data_element_id="3251",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
        ],
        segment_code="NAD",
        data_element_id="3207",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z03"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
        ],
        segment_code="NAD",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
        ],
        segment_code="NAD",
        data_element_id="3035",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
        ],
        segment_code="NAD",
        data_element_id="3036",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
        ],
        segment_code="NAD",
        data_element_id="3045",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
        ],
        segment_code="NAD",
        data_element_id="3045",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
        ],
        segment_code="NAD",
        data_element_id="3042",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
        ],
        segment_code="NAD",
        data_element_id="3164",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
        ],
        segment_code="NAD",
        data_element_id="3251",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
        ],
        segment_code="NAD",
        data_element_id="3207",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
        ],
        segment_code="RFF",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
        ],
        segment_code="RFF",
        data_element_id="1153",
        qualifier=None,
    ),
    AhbLocation(
        layers=[
            AhbLocationLayer(segment_group_key=None, opening_segment_code="UNH", opening_qualifier="UTILMD"),
            AhbLocationLayer(segment_group_key="SG4", opening_segment_code="IDE", opening_qualifier="24"),
            AhbLocationLayer(segment_group_key="SG12", opening_segment_code="NAD", opening_qualifier="Z05"),
        ],
        segment_code="RFF",
        data_element_id="1154",
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNT", opening_qualifier=None)],
        segment_code="UNT",
        data_element_id=None,
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNT", opening_qualifier=None)],
        segment_code="UNT",
        data_element_id="0074",
        qualifier=None,
    ),
    AhbLocation(
        layers=[AhbLocationLayer(segment_group_key=None, opening_segment_code="UNT", opening_qualifier=None)],
        segment_code="UNT",
        data_element_id="0062",
        qualifier=None,
    ),
]

example_flat_ahb_11042 = FlatAnwendungshandbuch(
    meta=AhbMetaInformation(pruefidentifikator="11042"),
    lines=[x[0] for x in lines_locations_changes_11042],
)
